@@ -22,16 +22,21 @@ import logging, os
 import numpy as np
 import numpy.ma as ma
 
-from argos.collect.collectortree import CollectorTree, CollectorSpinBox
+from argos.collect.collectortree import CollectorTree, CollectorSpinBox, SpinSlider
 from argos.inspector.abstract import UpdateReason
 from argos.qt import Qt, QtWidgets, QtGui, QtCore, QtSignal, QtSlot
 from argos.repo.baserti import BaseRti
-from argos.utils.cls import check_class, check_is_a_sequence, check_is_an_array, is_an_array
+from argos.utils.cls import check_class, check_is_a_sequence, check_is_an_array
 from argos.utils.masks import ArrayWithMask
+from argos.utils.moduleinfo import versionStrToTuple
 from argos.widgets.constants import TOP_DOCK_HEIGHT, DOCK_SPACING, DOCK_MARGIN
+from argos.widgets.misc import BasePanel
+
 
 logger = logging.getLogger(__name__)
 
+USE_SLIDER = True  # SpinSliders or only Spinboxes
+
 FAKE_DIM_NAME = '-'     # The name of the fake dimension with length 1
 FAKE_DIM_OFFSET = 1000  # Fake dimensions start here (so all arrays must have a smaller ndim)
 
@@ -41,14 +46,15 @@ FAKE_DIM_OFFSET = 1000  # Fake dimensions start here (so all arrays must have a
 
 
 
-class Collector(QtWidgets.QWidget):
+class Collector(BasePanel):
     """ Widget for collecting the selected data.
-        Consists of a tree to collect the VisItems, plus some buttons to add or remove then.
+        Consists of a table to collect the RtiItems
 
-        The CollectorTree only stores the VisItems, the intelligence is located in the Collector
+        The CollectorTree only stores the items, the intelligence is located in the Collector
         itself.
     """
     sigContentsChanged = QtSignal(str) # one of the UpdateReason values.
+    sigShowMessage = QtSignal(str)
 
     def __init__(self, windowNumber):
         """ Constructor
@@ -61,11 +67,12 @@ class Collector(QtWidgets.QWidget):
 
         self._signalsBlocked = False
         self.COL_FIRST_COMBO = 1     # Column that contains the first (left most) combobox
-        self.AXIS_POST_FIX = "-axis" # Added to the axis label to give the combox labels.
+        self.AXIS_POST_FIX = "-axis" # Added to the axis label to give the combobox labels.
         self._axisNames = []         # Axis names. Correspond to the independent variables
-        self._fullAxisNames = []       # Will be set in clearAndSetComboBoxes
+        self._fullAxisNames = []     # Will be set in clearAndSetComboBoxes
         self._comboBoxes = []        # Will be set in clearAndSetComboBoxes
         self._spinBoxes = []         # Will be set in createSpinBoxes
+        self._spinSliders = []       # Will be set in createSpinBoxes
 
         self.layout = QtWidgets.QHBoxLayout(self)
         self.layout.setSpacing(DOCK_SPACING)
@@ -168,7 +175,7 @@ class Collector(QtWidgets.QWidget):
 
 
     def clear(self):
-        """ Removes all VisItems
+        """ Removes all widgets
         """
         model = self.tree.model()
         # Don't use model.clear(). it will delete the column sizes
@@ -240,19 +247,25 @@ class Collector(QtWidgets.QWidget):
 
         # Create path label
         nodePath = '' if self.rti is None else self.rti.nodePath
-        pathItem = QtGui.QStandardItem(nodePath)
+        pathItem = model.item(row, 0)
+        if pathItem:
+            # Reusing path item. If we remove all items the column size will be lost.
+            pathItem.setText(nodePath)
+        else:
+            pathItem = QtGui.QStandardItem(nodePath)
+            pathItem.setEditable(False)
+            model.setItem(row, 0, pathItem)
+
         pathItem.setToolTip(nodePath)
-        pathItem.setEditable(False)
         if self.rti is not None:
             pathItem.setIcon(self.rti.decoration)
-        model.setItem(row, 0, pathItem)
 
         self._deleteSpinBoxes(row)
         self._populateComboBoxes(row)
         self._createSpinBoxes(row)
         self._updateRtiInfo()
 
-        self.tree.resizeColumnsToContents(startCol=self.COL_FIRST_COMBO)
+        self.tree.resizeColumnsFromContents(startCol=self.COL_FIRST_COMBO)
 
         logger.debug("{} sigContentsChanged signal (_updateWidgets)"
                       .format("Blocked" if self.signalsBlocked() else "Emitting"))
@@ -270,6 +283,7 @@ class Collector(QtWidgets.QWidget):
             logger.debug("Adding combobox at ({}, {})".format(row, col))
             comboBox = QtWidgets.QComboBox()
             comboBox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
+            comboBox.setMinimumContentsLength(3) # So that collector table header always readable
             comboBox.activated.connect(self._comboBoxActivated)
             self._comboBoxes.append(comboBox)
 
@@ -315,26 +329,44 @@ class Collector(QtWidgets.QWidget):
             for dimNr in range(nDims):
                 comboBox.addItem(self._rti.dimensionNames[dimNr], userData=dimNr)
 
-            # Set combobox current index
-            if nDims >= nCombos:
-                # We set the nth combo-box index to the last item - n. This because the
-                # NetCDF-CF conventions have the preferred dimension order of T, Z, Y, X.
-                # The +1 below is from the fake dimension.
-                curIdx = nDims + 1 - nCombos + comboBoxNr
-            else:
-                # If there are less dimensions in the RTI than the inspector can show, we fill
-                # the comboboxes starting at the leftmost and set the remaining comboboxes to the
-                # fake dimension. This means that a table inspector fill have one column and many
-                # rows, which is the most convenient.
-                curIdx = comboBoxNr + 1 if comboBoxNr < nDims else 0
-
-            assert 0 <= curIdx <= nDims + 1, \
-                "curIdx should be <= {}, got {}".format(nDims + 1, curIdx)
-
-            comboBox.setCurrentIndex(curIdx)
             comboBox.setEnabled(True)
             #comboBox.adjustSize() # necessary?
 
+        # Set combo-boxes current index
+        if nDims >= nCombos:
+            # If there are more dimensions in the RTI than the inspector can show, we use the
+            # last couple of RTI dimensions. This because the NetCDF-CF conventions have the
+            # preferred dimension order of T, Z, Y, X. On top of that, dimensions with length <= 1
+            # are skipped if possible (see Github issue #9).
+
+            curDim = nDims
+
+            for comboBoxNr, comboBox in reversed(list(enumerate(self._comboBoxes))):
+                while(True):
+                    curDim -= 1
+                    if curDim <= comboBoxNr or self.rti.arrayShape[curDim] > 1:
+                        break
+
+                curIdx = curDim + 1 # because of the fake dim
+                assert 0 <= curIdx <= nDims + 1, \
+                    "curIdx should be <= {}, got {}".format(nDims + 1, curIdx)
+
+                comboBox.setCurrentIndex(curIdx)
+        else:
+            # If there are less dimensions in the RTI than the inspector can show, we fill
+            # the combo-boxes starting at the leftmost and set the remaining comboboxes to the
+            # fake dimension. This means that a table inspector will have one column and many
+            # rows, which is the most convenient.
+
+            for comboBoxNr, comboBox in enumerate(self._comboBoxes):
+                curIdx = comboBoxNr + 1 if comboBoxNr < nDims else 0
+
+                assert 0 <= curIdx <= nDims + 1, \
+                    "curIdx should be <= {}, got {}".format(nDims + 1, curIdx)
+
+                comboBox.setCurrentIndex(curIdx)
+
+
 
     # def getComboBoxDimensionName(self, comboBoxNr):
     #     """ Returns the currently selected dimension index of combo box nr comboBoxNr
@@ -396,19 +428,30 @@ class Collector(QtWidgets.QWidget):
             spinBox.setMaximum(dimSize - 1)
             spinBox.setSingleStep(1)
             spinBox.setValue(dimSize // 2) # select the middle of the slice
-            spinBox.setPrefix("{}: ".format(self._rti.dimensionNames[dimNr]))
-            spinBox.setSuffix("/{}".format(spinBox.maximum()))
+            spinBox.setPrefix("{} = ".format(self._rti.dimensionNames[dimNr]))
             spinBox.setProperty("dim_nr", dimNr)
             #spinBox.adjustSize() # necessary?
 
+            if not USE_SLIDER:
+                spinBox.setSuffix("/{}".format(spinBox.maximum()))
+
+            if dimSize == 1:
+                spinBox.setEnabled(False)
+
             # This must be done after setValue to prevent emitting too many signals
             spinBox.valueChanged[int].connect(self._spinboxValueChanged)
 
-            tree.setIndexWidget(model.index(row, col), spinBox)
+            if USE_SLIDER:
+                spinSlider = SpinSlider(spinBox, layoutContentsMargins = (0, 0, 5, 0))
+                self._spinSliders.append(spinSlider)
+                tree.setIndexWidget(model.index(row, col), spinSlider)
+            else:
+                tree.setIndexWidget(model.index(row, col), spinBox)
+
             col += 1
 
         # Resize the spinbox columns to their new contents
-        self.tree.resizeColumnsToContents(startCol=self.COL_FIRST_COMBO + self.maxCombos)
+        self.tree.resizeColumnsFromContents(startCol=self.COL_FIRST_COMBO + self.maxCombos)
 
 
     def _deleteSpinBoxes(self, row):
@@ -472,6 +515,8 @@ class Collector(QtWidgets.QWidget):
             spinBox = self.sender()
         assert spinBox, "spinBox not defined and not the sender"
 
+        self._updateRtiInfo()
+
         logger.debug("{} sigContentsChanged signal (spinBox)"
                       .format("Blocked" if self.signalsBlocked() else "Emitting"))
         self.sigContentsChanged.emit(UpdateReason.COLLECTOR_SPIN_BOX)
@@ -486,14 +531,26 @@ class Collector(QtWidgets.QWidget):
                 the the slicedArray in your inspector! Note that this function calls transpose,
                 which can still make a copy of the array for certain permutations.
 
-            :return: Numpy masked array with the same number of dimension as the number of
+            :return: ArrayWithMask array with the same number of dimension as the number of
                 comboboxes (this can be zero!).
 
                 Returns None if no slice can be made (i.e. the RTI is not sliceable).
+            :rtype ArrayWithMask:
         """
         #logger.debug("getSlicedArray() called")
 
+        if not self._rti:
+            self.sigShowMessage.emit("No item selected.")
+            return None
+
         if not self.rtiIsSliceable:
+            # This is very common so we don't show a message so the user isn't flooded.
+            # Also this can have many different causes (compound data, lists, etc_ so it's
+            # difficult to show a good, descriptive message.
+            return None
+
+        if np.prod(self._rti.arrayShape) == 0:
+            self.sigShowMessage.emit("Selected item has zero array elements.")
             return None
 
         # The dimensions that are selected in the combo boxes will be set to slice(None),
@@ -509,19 +566,23 @@ class Collector(QtWidgets.QWidget):
         # interpreted as an index. With a tuple, array[(exp1, exp2, ..., expN)] is equivalent to
         # array[exp1, exp2, ..., expN].
         # See: http://docs.scipy.org/doc/numpy/reference/arrays.indexing.html
-        logger.debug("Array slice list: {}".format(str(sliceList)))
         slicedArray = self.rti[tuple(sliceList)]
 
         # Make a copy to prevent inspectors from modifying the underlying array.
         if copy:
-            slicedArray = ma.copy(slicedArray)
+            if versionStrToTuple(np.__version__) >= (1,19,0):
+                slicedArray = np.copy(slicedArray, subok=True)  # Fixes issue #8
+            else:
+                slicedArray = ma.copy(slicedArray)
 
         # If there are no comboboxes the sliceList will contain no Slices objects, only ints. Then
         # the resulting slicedArray will be a usually a scalar (only structured fields may yield an
         # array). We convert this scalar to a zero-dimensional Numpy array so that inspectors
         # always get an array (having the same number of dimensions as the dimensionality of the
         # inspector, i.e. the number of comboboxes).
-        if self.maxCombos == 0:
+        # Also scalar RTIs, which have nDim == 0, can return a scalar which must be converted.
+        # TODO: perhaps always convert to array.
+        if self.maxCombos == 0 or self.rti.nDims == 0:
             slicedArray = ma.MaskedArray(slicedArray)
 
         # Post-condition type check
@@ -532,6 +593,8 @@ class Collector(QtWidgets.QWidget):
             slicedArray = ma.MaskedArray(slicedArray)
 
         # Add fake dimensions of length 1 so that result.ndim will equal the number of combo boxes
+        # TODO: Perhaps get rid of this because it fails with masked arrays with fill values.
+        # The less we do here, the less chance an error occurs. See development/todo.txt
         for dimNr in range(slicedArray.ndim, self.maxCombos):
             #logger.debug("Adding fake dimension: {}".format(dimNr))
             slicedArray = ma.expand_dims(slicedArray, dimNr)
@@ -622,7 +685,7 @@ class Collector(QtWidgets.QWidget):
 
 
     def _updateRtiInfo(self):
-        """ Updates the _rtiInfo property when a new RTI is set or the comboboxes value change.
+        """ Updates the _rtiInfo property when a new RTI is set or spin/comboboxes change value .
         """
         logger.debug("Updating self._rtiInfo")
         # Info about the dependent dimension
