@@ -17,31 +17,47 @@
 
 """ Version and other info for this program
 """
-import os
+import os, sys
 
-DEBUGGING = False
+# We bypass the argparse mechanism in main.py because this import is executed before main.main()
+DEBUGGING = ('-d' in sys.argv or '--debug' in sys.argv)
+TESTING = True # add some test menu options
+PROFILING = False# and DEBUGGING
 
-VERSION = '0.2.2'
+VERSION = '0.3.1'
 REPO_NAME = "argos"
-#SCRIPT_NAME = "argos"
+SCRIPT_NAME = "argos"
 PACKAGE_NAME = "argos"
 PROJECT_NAME = "Argos"
-DEFAULT_PROFILE = 'Panoptes'
-SHORT_DESCRIPTION = "Argos Panoptes HDF/NCDF/scientific data viewer."
+SHORT_DESCRIPTION = "Argos HDF5/NetCDF/scientific data viewer."
 PROJECT_URL = "https://github.com/titusjan/argos"
 AUTHOR = "Pepijn Kenter"
 EMAIL = "titusjan@gmail.com"
 ORGANIZATION_NAME = "titusjan"
 ORGANIZATION_DOMAIN = "titusjan.nl"
 
+EXIT_CODE_SUCCESS = 0
+EXIT_CODE_ERROR = 1
+EXIT_CODE_COMMAND_ARGS = 2
+EXIT_CODE_RESTART = 66 # Indicates the program is being 'restarted'
 
+
+# TODO: move to utils locations
 def program_directory():
     """ Returns the program directory where this program is installed
     """
     return os.path.abspath(os.path.dirname(__file__))
 
+
+def resource_directory():
+    """ Returns directory with resources (images, style sheets, etc)
+    """
+    return os.path.join(program_directory(), 'img/')
+
+
+
 def icons_directory():
-    """ Returns the program directory where this program is installed
+    """ Returns the icons directory
     """
     return os.path.join(program_directory(), 'img/snipicons')
 
