@@ -19,51 +19,57 @@
 """
 from __future__ import division, print_function
 
+import os
 import logging, math
 import numpy as np
 import pyqtgraph as pg
 
 from functools import partial
 from collections import OrderedDict
+
 from argos.info import DEBUGGING
 from argos.config.boolcti import BoolCti, BoolGroupCti
 from argos.config.choicecti import ChoiceCti
 from argos.config.groupcti import MainGroupCti
 from argos.inspector.abstract import AbstractInspector, InvalidDataError, UpdateReason
-from argos.inspector.pgplugins.pgctis import (X_AXIS, Y_AXIS, BOTH_AXES, viewBoxAxisRange,
-                                                 defaultAutoRangeMethods, PgAxisLabelCti,
-                                                 PgAxisCti, PgAxisFlipCti, PgAspectRatioCti,
-                                                 PgAxisRangeCti, PgHistLutColorRangeCti, PgGridCti,
-                                                 PgGradientEditorItemCti, setXYAxesAutoRangeOn,
-                                                 PgPlotDataItemCti)
+from argos.inspector.pgplugins.colorbar import ArgosColorLegendItem
+from argos.inspector.pgplugins.pgctis import (
+    X_AXIS, Y_AXIS, BOTH_AXES, NO_LABEL_STR, defaultAutoRangeMethods, PgAxisLabelCti,
+    PgAxisCti, PgAxisFlipCti, PgAspectRatioCti, PgAxisRangeCti, PgGridCti,
+    PgColorMapCti, PgColorLegendCti, PgColorLegendLabelCti, PgShowHistCti,
+    PgShowDragLinesCti, setXYAxesAutoRangeOn, PgPlotDataItemCti)
 from argos.inspector.pgplugins.pgplotitem import ArgosPgPlotItem
-from argos.inspector.pgplugins.pghistlutitem import HistogramLUTItem
 from argos.qt import Qt, QtCore, QtGui, QtSlot
+
 from argos.utils.cls import array_has_real_numbers, check_class, is_an_array, to_string
-from argos.utils.masks import replaceMaskedValueWithFloat, maskedNanPercentile, ArrayWithMask
+from argos.utils.cls import array_kind_label
+from argos.utils.defs import RIGHT_ARROW
+from argos.utils.masks import (ArrayWithMask, replaceMaskedValueWithFloat,
+                               nanPercentileOfSubsampledArrayWithMask)
 
 logger = logging.getLogger(__name__)
 
 ROW_TITLE,    COL_TITLE    = 0, 0  # colspan = 3
-ROW_COLOR,    COL_COLOR    = 1, 0  # rowspan = 2
-ROW_HOR_LINE, COL_HOR_LINE = 1, 1
-ROW_IMAGE,    COL_IMAGE    = 2, 1
-ROW_VER_LINE, COL_VER_LINE = 2, 2
+ROW_COLOR,    COL_COLOR    = 1, 2  # rowspan = 2
+ROW_HOR_LINE, COL_HOR_LINE = 1, 0
+ROW_IMAGE,    COL_IMAGE    = 2, 0
+ROW_VER_LINE, COL_VER_LINE = 2, 1
 ROW_PROBE,    COL_PROBE    = 3, 0  # colspan = 2
 
 
-
-def calcPgImagePlot2dDataRange(pgImagePlot2d, percentage, crossPlot):
+def calcPgImagePlot2dDataRange(pgImagePlot2d, percentage, crossPlot, subsample):
     """ Calculates the range from the inspectors' sliced array. Discards percentage of the minimum
         and percentage of the maximum values of the inspector.slicedArray
 
         :param pgImagePlot2d: the range methods will work on (the sliced array) of this inspector.
-        :param percentage: percentage that will be discarded.
-        :param crossPlot: if None, the range will be calculated from the entire sliced array,
+        :param float percentage: percentage that will be discarded.
+        :param bool crossPlot: if None, the range will be calculated from the entire sliced array,
             if "horizontal" or "vertical" the range will be calculated from the data under the
             horizontal or vertical cross hairs.
             If the cursor is outside the image, there is no valid data under the cross-hair and
             the range will be determined from the sliced array as a fall back.
+        :param bool subsample: if True, the image will be subsampled (to 200 by 200) before
+            calculating the range. This to improve performance by large images.
     """
     check_class(pgImagePlot2d.slicedArray, ArrayWithMask) # sanity check
 
@@ -72,20 +78,20 @@ def calcPgImagePlot2dDataRange(pgImagePlot2d, percentage, crossPlot):
 
     elif crossPlot == 'horizontal':
         if pgImagePlot2d.crossPlotRow is not None:
-            array = pgImagePlot2d.slicedArray.asMaskedArray()[pgImagePlot2d.crossPlotRow, :]
+            array = pgImagePlot2d.slicedArray[pgImagePlot2d.crossPlotRow, :]
         else:
             array = pgImagePlot2d.slicedArray # fall back on complete sliced array
 
     elif crossPlot == 'vertical':
         if pgImagePlot2d.crossPlotCol is not None:
-            array = pgImagePlot2d.slicedArray.asMaskedArray()[:, pgImagePlot2d.crossPlotCol]
+            array = pgImagePlot2d.slicedArray[:, pgImagePlot2d.crossPlotCol]
         else:
             array = pgImagePlot2d.slicedArray # fall back on complete sliced array
     else:
         raise ValueError("crossPlot must be: None, 'horizontal' or 'vertical', got: {}"
                          .format(crossPlot))
 
-    return maskedNanPercentile(array, (percentage, 100-percentage) )
+    return nanPercentileOfSubsampledArrayWithMask(array, (percentage, 100 - percentage), subsample)
 
 
 def crossPlotAutoRangeMethods(pgImagePlot2d, crossPlot, intialItems=None):
@@ -137,72 +143,93 @@ class PgImagePlot2dCti(MainGroupCti):
         imagePlotItem = self.pgImagePlot2d.imagePlotItem
         viewBox = imagePlotItem.getViewBox()
 
-        self.insertChild(ChoiceCti('title', 0, editable=True,
-                                   configValues=["{base-name} -- {name} {slices}",
-                                                 "{name} {slices}", "{path} {slices}"]))
+        self.insertChild(
+            ChoiceCti('title', 0, editable=True,
+                      configValues=["{base-name} -- {name} {slices}",
+                                    "{name} {slices}", "{path} {slices}"]))
+
         #### Axes ####
+
         self.aspectLockedCti = self.insertChild(PgAspectRatioCti(viewBox))
 
         self.xAxisCti = self.insertChild(PgAxisCti('x-axis'))
-        #xAxisCti.insertChild(PgAxisShowCti(imagePlotItem, 'bottom')) # disabled, seems broken
-        self.xAxisCti.insertChild(PgAxisLabelCti(imagePlotItem, 'bottom', self.pgImagePlot2d.collector,
-            defaultData=1, configValues=[PgAxisLabelCti.NO_LABEL, "{x-dim} [index]"]))
+        self.xAxisCti.insertChild(
+            PgAxisLabelCti(imagePlotItem, 'bottom', self.pgImagePlot2d.collector,
+                           defaultData=1,
+                           configValues=[NO_LABEL_STR, "{x-dim} [index]"]))
         self.xFlippedCti = self.xAxisCti.insertChild(PgAxisFlipCti(viewBox, X_AXIS))
         self.xAxisRangeCti = self.xAxisCti.insertChild(PgAxisRangeCti(viewBox, X_AXIS))
 
         self.yAxisCti = self.insertChild(PgAxisCti('y-axis'))
-        #yAxisCti.insertChild(PgAxisShowCti(imagePlotItem, 'left'))  # disabled, seems broken
-        self.yAxisCti.insertChild(PgAxisLabelCti(imagePlotItem, 'left', self.pgImagePlot2d.collector,
-            defaultData=1, configValues=[PgAxisLabelCti.NO_LABEL, "{y-dim} [index]"]))
-        self.yFlippedCti = self.yAxisCti.insertChild(PgAxisFlipCti(viewBox, Y_AXIS))
+        self.yAxisCti.insertChild(
+            PgAxisLabelCti(imagePlotItem, 'left', self.pgImagePlot2d.collector,
+                           defaultData=1,
+                           configValues=[NO_LABEL_STR, "{y-dim} [index]"]))
+        self.yFlippedCti = self.yAxisCti.insertChild(
+            PgAxisFlipCti(viewBox, Y_AXIS, defaultData=True))
         self.yAxisRangeCti = self.yAxisCti.insertChild(PgAxisRangeCti(viewBox, Y_AXIS))
 
         #### Color scale ####
 
+        self.colorCti = self.insertChild(PgAxisCti('color scale'))
+
+        self.colorCti.insertChild(PgColorLegendLabelCti(
+            pgImagePlot2d.colorLegendItem, self.pgImagePlot2d.collector, defaultData=1,
+            configValues=[NO_LABEL_STR, "{name} {unit}", "{path} {unit}",
+                          "{name}", "{path}", "{raw-unit}"]))
+
+        self.colorCti.insertChild(PgColorMapCti(self.pgImagePlot2d.colorLegendItem))
+
+        self.showHistCti = self.colorCti.insertChild(
+            PgShowHistCti(pgImagePlot2d.colorLegendItem))
+        self.showDragLinesCti = self.colorCti.insertChild(
+            PgShowDragLinesCti(pgImagePlot2d.colorLegendItem))
+
         colorAutoRangeFunctions = defaultAutoRangeMethods(self.pgImagePlot2d)
+        self.colorLegendCti = self.colorCti.insertChild(
+            PgColorLegendCti(pgImagePlot2d.colorLegendItem, colorAutoRangeFunctions,
+                             nodeName="range"))
 
-        self.histColorRangeCti = self.insertChild(
-            PgHistLutColorRangeCti(pgImagePlot2d.histLutItem, colorAutoRangeFunctions,
-                                   nodeName="color range"))
+        # If True, the image is automatically downsampled to match the screen resolution. This
+        # improves performance for large images and reduces aliasing. If autoDownsample is not
+        # specified, then ImageItem will choose whether to downsample the image based on its size.
+        self.autoDownSampleCti = self.insertChild(BoolCti('auto down sample', True))
+        self.zoomModeCti = self.insertChild(BoolCti('rectangle zoom mode', False))
 
-        histViewBox = pgImagePlot2d.histLutItem.vb
-        histViewBox.enableAutoRange(Y_AXIS, False)
-        rangeFunctions = defaultAutoRangeMethods(self.pgImagePlot2d,
-            {PgAxisRangeCti.PYQT_RANGE: partial(viewBoxAxisRange, histViewBox, Y_AXIS)})
+        ### Probe and cross-hair plots ###
 
-        self.histRangeCti = self.insertChild(
-            PgAxisRangeCti(histViewBox, Y_AXIS, nodeName='histogram range',
-                           autoRangeFunctions=rangeFunctions))
-
-        self.insertChild(PgGradientEditorItemCti(self.pgImagePlot2d.histLutItem.gradient))
-
-        # Probe and cross-hair plots
         self.probeCti = self.insertChild(BoolCti('show probe', True))
-
-        self.crossPlotGroupCti = self.insertChild(BoolGroupCti('cross-hair',  expanded=False))
-
+        self.crossPlotGroupCti = self.insertChild(BoolGroupCti('cross-hair', expanded=False))
         self.crossPenCti = self.crossPlotGroupCti.insertChild(PgPlotDataItemCti(expanded=False))
-        #self.crossLinesCti = self.crossPlotGroupCti.insertChild(PgPlotDataItemCti('cross pen',
-        #                                                                          expanded=False))
 
-        self.horCrossPlotCti = self.crossPlotGroupCti.insertChild(BoolCti('horizontal', False,
-                                                                          expanded=False))
+        self.horCrossPlotCti = self.crossPlotGroupCti.insertChild(
+            BoolCti('horizontal', False, expanded=False))
+
         self.horCrossPlotCti.insertChild(PgGridCti(pgImagePlot2d.horCrossPlotItem))
-        self.horCrossPlotRangeCti = self.horCrossPlotCti.insertChild(PgAxisRangeCti(
-            self.pgImagePlot2d.horCrossPlotItem.getViewBox(), Y_AXIS, nodeName="data range",
-            autoRangeFunctions = crossPlotAutoRangeMethods(self.pgImagePlot2d, "horizontal")))
+        self.horCrossPlotRangeCti = self.horCrossPlotCti.insertChild(
+            PgAxisRangeCti(
+                self.pgImagePlot2d.horCrossPlotItem.getViewBox(), Y_AXIS, nodeName="data range",
+                autoRangeFunctions=crossPlotAutoRangeMethods(self.pgImagePlot2d, "horizontal")))
 
-        self.verCrossPlotCti = self.crossPlotGroupCti.insertChild(BoolCti('vertical', False,
-                                                                          expanded=False))
+        self.verCrossPlotCti = self.crossPlotGroupCti.insertChild(
+            BoolCti('vertical', False, expanded=False))
         self.verCrossPlotCti.insertChild(PgGridCti(pgImagePlot2d.verCrossPlotItem))
-        self.verCrossPlotRangeCti = self.verCrossPlotCti.insertChild(PgAxisRangeCti(
-            self.pgImagePlot2d.verCrossPlotItem.getViewBox(), X_AXIS, nodeName="data range",
-            autoRangeFunctions = crossPlotAutoRangeMethods(self.pgImagePlot2d, "vertical")))
+        self.verCrossPlotRangeCti = self.verCrossPlotCti.insertChild(
+            PgAxisRangeCti(
+                self.pgImagePlot2d.verCrossPlotItem.getViewBox(), X_AXIS, nodeName="data range",
+                autoRangeFunctions=crossPlotAutoRangeMethods(self.pgImagePlot2d, "vertical")))
 
-        # Connect signals
-        self.pgImagePlot2d.imagePlotItem.sigAxisReset.connect(self.setImagePlotAutoRangeOn)
-        self.pgImagePlot2d.horCrossPlotItem.sigAxisReset.connect(self.setHorCrossPlotAutoRangeOn)
-        self.pgImagePlot2d.verCrossPlotItem.sigAxisReset.connect(self.setVerCrossPlotAutoRangeOn)
+        # Connect signals.
+
+        # Use a queued connect to schedule the reset after current events have been processed.
+        self.pgImagePlot2d.colorLegendItem.sigResetColorScale.connect(
+            self.colorLegendCti.setAutoRangeOn, type=Qt.QueuedConnection)
+        self.pgImagePlot2d.imagePlotItem.sigResetAxis.connect(
+            self.setImagePlotAutoRangeOn, type=Qt.QueuedConnection)
+        self.pgImagePlot2d.horCrossPlotItem.sigResetAxis.connect(
+            self.setHorCrossPlotAutoRangeOn, type=Qt.QueuedConnection)
+        self.pgImagePlot2d.verCrossPlotItem.sigResetAxis.connect(
+            self.setVerCrossPlotAutoRangeOn, type=Qt.QueuedConnection)
 
         # Also update axis auto range tree items when linked axes are resized
         horCrossViewBox = self.pgImagePlot2d.horCrossPlotItem.getViewBox()
@@ -215,14 +242,17 @@ class PgImagePlot2dCti(MainGroupCti):
         """ Disconnects signals.
             Is called by self.finalize when the cti is deleted.
         """
+        self.pgImagePlot2d.colorLegendItem.sigResetColorScale.disconnect(
+            self.colorLegendCti.setAutoRangeOn)
+
         verCrossViewBox = self.pgImagePlot2d.verCrossPlotItem.getViewBox()
         verCrossViewBox.sigRangeChangedManually.disconnect(self.yAxisRangeCti.setAutoRangeOff)
         horCrossViewBox = self.pgImagePlot2d.horCrossPlotItem.getViewBox()
         horCrossViewBox.sigRangeChangedManually.disconnect(self.xAxisRangeCti.setAutoRangeOff)
 
-        self.pgImagePlot2d.verCrossPlotItem.sigAxisReset.disconnect(self.setVerCrossPlotAutoRangeOn)
-        self.pgImagePlot2d.horCrossPlotItem.sigAxisReset.disconnect(self.setHorCrossPlotAutoRangeOn)
-        self.pgImagePlot2d.imagePlotItem.sigAxisReset.disconnect(self.setImagePlotAutoRangeOn)
+        self.pgImagePlot2d.verCrossPlotItem.sigResetAxis.disconnect(self.setVerCrossPlotAutoRangeOn)
+        self.pgImagePlot2d.horCrossPlotItem.sigResetAxis.disconnect(self.setHorCrossPlotAutoRangeOn)
+        self.pgImagePlot2d.imagePlotItem.sigResetAxis.disconnect(self.setImagePlotAutoRangeOn)
 
 
     def setImagePlotAutoRangeOn(self, axisNumber):
@@ -249,9 +279,21 @@ class PgImagePlot2dCti(MainGroupCti):
         setXYAxesAutoRangeOn(self, self.verCrossPlotRangeCti, self.yAxisRangeCti, axisNumber)
 
 
+    def resetRangesToDefault(self):
+        """ Resets range settings to the default data.
+        """
+        self.xAxisRangeCti.autoRangeCti.data = True
+        self.yAxisRangeCti.autoRangeCti.data = True
+        self.colorLegendCti.autoRangeCti.data = True
+        self.horCrossPlotRangeCti.autoRangeCti.data = True
+        self.verCrossPlotRangeCti.autoRangeCti.data = True
+
+
 
 class PgImagePlot2d(AbstractInspector):
-    """ Inspector that contains a PyQtGraph 2-dimensional image plot.
+    """ Draws an image plot of a two-dimensional array (slice).
+
+        Plotting is done with the PyQtGraph package. See www.pyqtgraph.org.
     """
 
     def __init__(self, collector, parent=None):
@@ -272,12 +314,10 @@ class PgImagePlot2d(AbstractInspector):
         self.viewBox.disableAutoRange(BOTH_AXES)
 
         self.imageItem = pg.ImageItem()
+        self.imageItem.setPos(-0.5, -0.5) # Center on pixels (see pg.ImageView.setImage source code)
         self.imagePlotItem.addItem(self.imageItem)
 
-        self.histLutItem = HistogramLUTItem() # what about GradientLegend?
-        self.histLutItem.setImageItem(self.imageItem)
-        self.histLutItem.vb.setMenuEnabled(False)
-        self.histLutItem.setHistogramRange(0, 100) # Disables autoscaling
+        self.colorLegendItem = ArgosColorLegendItem(self.imageItem)
 
         # Probe and cross hair plots
         self.crossPlotRow = None # the row coordinate of the cross hair. None if no cross hair.
@@ -319,7 +359,7 @@ class PgImagePlot2d(AbstractInspector):
         self.contentsLayout.addWidget(self.graphicsLayoutWidget)
 
         self.graphicsLayoutWidget.addItem(self.titleLabel, ROW_TITLE, COL_TITLE, colspan=3)
-        self.graphicsLayoutWidget.addItem(self.histLutItem, ROW_COLOR, COL_COLOR, rowspan=2)
+        self.graphicsLayoutWidget.addItem(self.colorLegendItem, ROW_COLOR, COL_COLOR, rowspan=2)
         self.graphicsLayoutWidget.addItem(self.imagePlotItem, ROW_IMAGE, COL_IMAGE)
         self.graphicsLayoutWidget.addItem(self.probeLabel, ROW_PROBE, COL_PROBE, colspan=3)
 
@@ -346,6 +386,7 @@ class PgImagePlot2d(AbstractInspector):
         """ Is called before destruction. Can be used to clean-up resources.
         """
         logger.debug("Finalizing: {}".format(self))
+        self.colorLegendItem.finalize()
         self.imagePlotItem.scene().sigMouseMoved.disconnect(self.mouseMoved)
         self.imagePlotItem.close()
         self.graphicsLayoutWidget.close()
@@ -359,27 +400,28 @@ class PgImagePlot2d(AbstractInspector):
         return tuple(['Y', 'X'])
 
 
-    def _hasValidData(self):
-        """ Returns True if the inspector has data that can be plotted.
-        """
-        return self.slicedArray is not None and array_has_real_numbers(self.slicedArray.data)
-
-
     def _clearContents(self):
         """ Clears the contents when no valid data is available
         """
         logger.debug("Clearing inspector contents")
+        self.slicedArray = None
         self.titleLabel.setText('')
 
         # Don't clear the imagePlotItem, the imageItem is only added in the constructor.
         self.imageItem.clear()
+        if hasattr(self.imageItem, '_wasIntegerData'):
+            del self.imageItem._wasIntegerData
+
+        # Unfortunately PyQtGraph doesn't emit this signal when the image is cleared.
+        self.imageItem.sigImageChanged.emit()
+
         self.imagePlotItem.setLabel('left', '')
         self.imagePlotItem.setLabel('bottom', '')
-
+        self.colorLegendItem.setLabel('')
         # Set the histogram range and levels to finite values to prevent futher errors if this
         # function was called after an exception in self.drawContents
-        self.histLutItem.setHistogramRange(0, 100)
-        self.histLutItem.setLevels(0, 100)
+        #self.histLutItem.setHistogramRange(0, 100)
+        #self.histLutItem.setLevels(0, 100)
 
         self.crossPlotRow, self.crossPlotCol = None, None
 
@@ -399,6 +441,11 @@ class PgImagePlot2d(AbstractInspector):
             The reason parameter is used to determine if the axes will be reset (the initiator
             parameter is ignored). See AbstractInspector.updateContents for their description.
         """
+
+        # If auto-reset is true, reset config complete or partially, depending on the mode.
+        if self._resetRequired(reason, initiator):
+            self.resetConfig()
+
         self.crossPlotRow = None # reset because the sliced array shape may change
         self.crossPlotCol = None # idem dito
 
@@ -430,15 +477,32 @@ class PgImagePlot2d(AbstractInspector):
                 self.verPlotAdded = False
                 gridLayout.activate()
 
-        self.slicedArray = self.collector.getSlicedArray()
-
-        if not self._hasValidData():
+        slicedArray = self.collector.getSlicedArray()
+        if slicedArray is None:
             self._clearContents()
-            raise InvalidDataError("No data available or it does not contain real numbers")
+            raise InvalidDataError()  # Don't show message, to common.
+        elif not array_has_real_numbers(slicedArray.data):
+            self._clearContents()
+            raise InvalidDataError(
+                "Selected item contains {} data.".format(array_kind_label(slicedArray.data)))
+        else:
+            self.slicedArray = slicedArray
 
         # -- Valid plot data from here on --
 
-        # PyQtGraph doesn't handle masked array so we convert the masked values to Nans. Missing
+        if self.config.crossPlotGroupCti.checkState != Qt.Unchecked:
+            tempPlotDataItem = self.config.crossPenCti.createPlotDataItem()
+            if tempPlotDataItem.opts['pen'] is None and tempPlotDataItem.opts['symbol'] is None:
+                self.sigShowMessage.emit(
+                    "The cross-hair pen 'line' and 'symbol' config options are both unchecked!")
+
+        numElem = np.prod(self.slicedArray.data.shape)
+        if numElem == 0:
+            self.sigShowMessage.emit("Current slice is empty.")  # Not expected to happen.
+        elif numElem == 1:
+            self.sigShowMessage.emit("Current slice contains only a single data point.")
+
+        # PyQtGraph doesn't handle masked arrays so we convert the masked values to Nans. Missing
         # data values are replaced by NaNs. The PyQtGraph image plot shows this as the color at the
         # lowest end of the color scale. Unfortunately we cannot choose a missing-value color, but
         # at least the Nans do not influence for the histogram and color range.
@@ -447,7 +511,7 @@ class PgImagePlot2d(AbstractInspector):
         imageArray = replaceMaskedValueWithFloat(self.slicedArray.data, self.slicedArray.mask,
                                                  np.nan, copyOnReplace=True)
 
-        # Replace infinite value with Nans because PyQtGraph fails on them. WNote that the CTIs of
+        # Replace infinite value with Nans because PyQtGraph fails on them. Note that the CTIs of
         # the cross plots (e.g. horCrossPlotRangeCti) are still connected to self.slicedArray, so
         # if the cross section consists of only infs, they may not able to update the autorange.
         # A warning is issued in that case.
@@ -456,21 +520,23 @@ class PgImagePlot2d(AbstractInspector):
         imageArray = replaceMaskedValueWithFloat(imageArray, np.isinf(self.slicedArray.data),
                                                  np.nan, copyOnReplace=True)
 
-        # Reset the axes ranges (via the config)
-        if (reason == UpdateReason.RTI_CHANGED or
-            reason == UpdateReason.COLLECTOR_COMBO_BOX):
-            self.config.xAxisRangeCti.autoRangeCti.data = True
-            self.config.yAxisRangeCti.autoRangeCti.data = True
-            self.config.histColorRangeCti.autoRangeCti.data = True
-            self.config.histRangeCti.autoRangeCti.data = True
-            self.config.horCrossPlotRangeCti.autoRangeCti.data = True
-            self.config.verCrossPlotRangeCti.autoRangeCti.data = True
-
         # PyQtGraph uses the following dimension order: T, X, Y, Color.
         # We need to transpose the slicedArray ourselves because axes = {'x':1, 'y':0}
         # doesn't seem to do anything.
         imageArray = imageArray.transpose()
-        self.imageItem.setImage(imageArray, autoLevels=False)
+
+        # Set the _wasIntegerData to True if the original data type was a signed or unsigned. This
+        # allows the ArgosColorLegendItem to make histogram bins as if it were an integer
+        self.imageItem._wasIntegerData = self.slicedArray.data.dtype.kind in 'ui'
+        self.imageItem.setAutoDownsample(self.config.autoDownSampleCti.configValue)
+        self.imageItem.setImage(imageArray, autoLevels=False)  # Do after _wasIntegerData is set!
+
+        self.imagePlotItem.setRectangleZoomOn(self.config.zoomModeCti.configValue)
+
+        # Always use pan mode in the cross plots. Rectangle zoom is akward there and it's nice to
+        # still be able to pan.
+        #self.horCrossPlotItem.setRectangleZoomOn(self.config.zoomModeCti.configValue)
+        #self.verCrossPlotItem.setRectangleZoomOn(self.config.zoomModeCti.configValue)
 
         self.horCrossPlotItem.invertX(self.config.xFlippedCti.configValue)
         self.verCrossPlotItem.invertY(self.config.yFlippedCti.configValue)
@@ -479,8 +545,7 @@ class PgImagePlot2d(AbstractInspector):
 
         self.titleLabel.setText(self.configValue('title').format(**self.collector.rtiInfo))
 
-        # Update the config tree from the (possibly) new state of the PgImagePlot2d inspector,
-        # e.g. the axis range or color range may have changed while drawing.
+        # self.config.logBranch()
         self.config.updateTarget()
 
 
@@ -494,7 +559,7 @@ class PgImagePlot2d(AbstractInspector):
             show_data_point = False # shows the data point as a circle in the cross hair plots
             self.crossPlotRow, self.crossPlotCol = None, None
 
-            self.probeLabel.setText("<span style='color: #808080'>no data at cursor</span>")
+            self.probeLabel.setText("<span style='color: #808080'>No data at cursor</span>")
             self.crossLineHorizontal.setVisible(False)
             self.crossLineVertical.setVisible(False)
             self.crossLineHorShadow.setVisible(False)
@@ -503,13 +568,11 @@ class PgImagePlot2d(AbstractInspector):
             self.horCrossPlotItem.clear()
             self.verCrossPlotItem.clear()
 
-            if (self._hasValidData() and self.slicedArray is not None
-                and self.viewBox.sceneBoundingRect().contains(viewPos)):
+            if self.slicedArray is not None and self.viewBox.sceneBoundingRect().contains(viewPos):
 
-                # Calculate the row and column at the cursor. We use math.floor because the pixel
-                # corners of the image lie at integer values (and not the centers of the pixels).
+                # Calculate the row and column at the cursor.
                 scenePos = self.viewBox.mapSceneToView(viewPos)
-                row, col = math.floor(scenePos.y()), math.floor(scenePos.x())
+                row, col = round(scenePos.y()), round(scenePos.x())
                 row, col = int(row), int(col) # Needed in Python 2
                 nRows, nCols = self.slicedArray.shape
 
@@ -518,10 +581,13 @@ class PgImagePlot2d(AbstractInspector):
 
                     self.crossPlotRow, self.crossPlotCol = row, col
                     index = tuple([row, col])
-                    valueStr = to_string(self.slicedArray[index],
+                    valueStr = to_string(self.slicedArray.data[index],
                                          masked=self.slicedArray.maskAt(index),
                                          maskFormat='&lt;masked&gt;')
-                    txt = "pos = ({:d}, {:d}), value = {}".format(row, col, valueStr)
+
+                    txt = "({}, {}) = ({:d}, {:d}) {} {} = {}".format(
+                        self.collector.rtiInfo['x-dim'], self.collector.rtiInfo['y-dim'],
+                        col, row, RIGHT_ARROW, self.collector.rtiInfo['name'], valueStr)
                     self.probeLabel.setText(txt)
 
                     # Show cross section at the cursor pos in the line plots
@@ -541,14 +607,24 @@ class PgImagePlot2d(AbstractInspector):
                             connected = (np.zeros_like(rowData)
                                          if self.slicedArray.mask else connected)
 
+                        # Replace mask by Nans. Only doing when not showing lines to hack around PyQtGraph issue 1057
+                        # See comment in PgLinePlot1d._drawContents for a more detailed explanation
+                        # TODO: reuse imageItem data array when this hack is no longer necessary
+                        if not self.config.crossPenCti.lineCti.configValue:
+                            rowData = replaceMaskedValueWithFloat(rowData, np.logical_not(connected),
+                                                                  np.nan, copyOnReplace=True)
+
                         # Replace infinite value with nans because PyQtGraph can't handle them
                         rowData = replaceMaskedValueWithFloat(rowData, np.isinf(rowData),
                                                               np.nan, copyOnReplace=True)
 
                         horPlotDataItem = self.config.crossPenCti.createPlotDataItem()
+                        # TODO: try to use connect='finite' when the hack above is no longer necessary. In that case
+                        # test with array_masked test data
                         horPlotDataItem.setData(rowData, connect=connected)
                         self.horCrossPlotItem.addItem(horPlotDataItem)
 
+
                         # Vertical line in hor-cross plot
                         crossLineShadow90 = pg.InfiniteLine(angle=90, movable=False,
                                                             pen=self.crossShadowPen)
@@ -560,7 +636,8 @@ class PgImagePlot2d(AbstractInspector):
 
                         if show_data_point:
                             crossPoint90 = pg.PlotDataItem(symbolPen=self.crossPen)
-                            crossPoint90.setSymbolBrush(QtGui.QBrush(self.config.crossPenCti.penColor))
+                            crossPoint90.setSymbolBrush(QtGui.QBrush(
+                                    self.config.crossPenCti.penColor))
                             crossPoint90.setSymbolSize(10)
                             crossPoint90.setData((col,), (rowData[col],))
                             self.horCrossPlotItem.addItem(crossPoint90, ignoreBounds=True)
@@ -584,6 +661,12 @@ class PgImagePlot2d(AbstractInspector):
                             connected = (np.zeros_like(colData)
                                          if self.slicedArray.mask else connected)
 
+                        # Replace mask by Nans. Only doing when not showing lines to hack around PyQtGraph issue 1057
+                        # See comment in PgLinePlot1d._drawContents for a more detailed explanation
+                        if not self.config.crossPenCti.lineCti.configValue:
+                            colData = replaceMaskedValueWithFloat(colData, np.logical_not(connected),
+                                                                  np.nan, copyOnReplace=True)
+
                         # Replace infinite value with nans because PyQtGraph can't handle them
                         colData = replaceMaskedValueWithFloat(colData, np.isinf(colData),
                                                               np.nan, copyOnReplace=True)
