@@ -27,20 +27,31 @@ import pyqtgraph as pg
 # http://docs.python-guide.org/en/latest/writing/gotchas/#late-binding-closures
 from functools import partial
 from collections import OrderedDict
+
+from cmlib import ColorSelectionWidget, ColorMap, makeColorBarPixmap
+from cmlib import CmMetaData, CatalogMetaData
+
 from argos.config.groupcti import GroupCti
+from argos.config.abstractcti import AbstractCti, AbstractCtiEditor
 from argos.config.boolcti import BoolCti, BoolGroupCti
 from argos.config.choicecti import ChoiceCti
 from argos.config.intcti import IntCti
 from argos.config.floatcti import SnFloatCti, FloatCti
 from argos.config.qtctis import PenCti, ColorCti, createPenStyleCti, createPenWidthCti
 from argos.config.untypedcti import UntypedCti
+from argos.inspector.pgplugins.colorbar import ArgosColorLegendItem
 from argos.inspector.pgplugins.pghistlutitem import HistogramLUTItem
-from argos.qt import QtGui, QtWidgets
+from argos.qt import QtCore, QtGui, QtWidgets, QtSlot
+from argos.qt.misc import setWidgetSizePolicy
+from argos.repo.colors import CmLibModelSingleton, DEFAULT_COLOR_MAP
 from argos.utils.cls import check_class
-from argos.utils.masks import maskedNanPercentile
+from argos.utils.masks import nanPercentileOfSubsampledArrayWithMask
+
 
 from pyqtgraph.graphicsItems.GradientEditorItem import Gradients as GRADIENTS
 
+from pgcolorbar.colorlegend import ColorLegendItem
+
 logger = logging.getLogger(__name__)
 
 X_AXIS = pg.ViewBox.XAxis
@@ -48,7 +59,7 @@ Y_AXIS = pg.ViewBox.YAxis
 BOTH_AXES = pg.ViewBox.XYAxes
 VALID_AXIS_NUMBERS = (X_AXIS, Y_AXIS, BOTH_AXES)
 VALID_AXIS_POSITIONS =  ('left', 'right', 'bottom', 'top')
-
+NO_LABEL_STR = '-- none --'
 
 
 class ViewBoxDebugCti(GroupCti):
@@ -126,7 +137,7 @@ def viewBoxAxisRange(viewBox, axisNumber):
         raise AssertionError("No children bbox. Plot range not updated.")
 
 
-def inspectorDataRange(inspector, percentage):
+def inspectorDataRange(inspector, percentage, subsample):
     """ Calculates the range from the inspectors' sliced array. Discards percentage of the minimum
         and percentage of the maximum values of the inspector.slicedArray
 
@@ -134,8 +145,10 @@ def inspectorDataRange(inspector, percentage):
         The first parameter is an inspector, it's not an array, because we would then have to
         regenerate the range function every time sliced array of an inspector changes.
     """
-    logger.debug("Discarding {}% from id: {}".format(percentage, id(inspector.slicedArray)))
-    return maskedNanPercentile(inspector.slicedArray, (percentage, 100-percentage) )
+    logger.debug("Discarding {}% from id: 0x{:08x}".format(percentage, id(inspector.slicedArray)))
+
+    return nanPercentileOfSubsampledArrayWithMask(
+        inspector.slicedArray, (percentage, 100-percentage), subsample)
 
 
 def defaultAutoRangeMethods(inspector, intialItems=None):
@@ -186,7 +199,8 @@ class AbstractRangeCti(GroupCti):
 
         Is an abstract class. Descendants must override getTargetRange and setTargetRange
     """
-    def __init__(self, autoRangeFunctions=None, nodeName='range', expanded=False):
+    def __init__(self, autoRangeFunctions=None, nodeName='range',
+                 expanded=False, paddingDefault=-1):
         """ Constructor.
             The target axis is specified by viewBox and axisNumber (0 for x-axis, 1 for y-axis)
 
@@ -195,12 +209,16 @@ class AbstractRangeCti(GroupCti):
             If autoRangeFunctions is None, there will be no auto-range child CTI.
             If autoRangeFunctions has one element there will be an auto-range child without a method
             child CTI (the function from the autoRangeMethods dictionary will be the default).
+
+            :param int paddingDefault: default padding value. Use -1 for 'dynamic' padding, which
+                is the PyQtGraph padding algorithm.
         """
         super(AbstractRangeCti, self).__init__(nodeName, expanded=expanded)
 
         self._rangeFunctions = {}
         self.autoRangeCti = None
         self.methodCti = None
+        self.subsampleCti = None
         self.paddingCti = None
 
         if autoRangeFunctions is not None:
@@ -211,7 +229,11 @@ class AbstractRangeCti(GroupCti):
                 self.methodCti = ChoiceCti("method", configValues=list(autoRangeFunctions.keys()))
                 self.autoRangeCti.insertChild(self.methodCti)
 
-            self.paddingCti = IntCti("padding", -1, suffix="%", specialValueText="dynamic",
+                self.subsampleCti = BoolCti("subsample", True)
+                self.autoRangeCti.insertChild(self.subsampleCti)
+
+            self.paddingCti = IntCti("padding", paddingDefault,
+                                     suffix="%", specialValueText="dynamic",
                                      minValue=-1, maxValue=1000, stepSize=1)
             self.autoRangeCti.insertChild(self.paddingCti)
 
@@ -248,7 +270,7 @@ class AbstractRangeCti(GroupCti):
     def _forceRefreshMinMax(self):
         """ Refreshes the min max config values from the axes' state.
         """
-        #logger.debug("_forceRefreshMinMax", stack_info=True)
+        #logger.debug("_forceRefreshMinMax")
 
         # Set the precision from by looking how many decimals are needed to show the difference
         # between the minimum and maximum, given the maximum. E.g. if min = 0.04 and max = 0.07,
@@ -335,7 +357,11 @@ class AbstractRangeCti(GroupCti):
             return (self.rangeMinCti.data, self.rangeMaxCti.data)
         else:
             rangeFunction = self._rangeFunctions[self.autoRangeMethod]
-            return rangeFunction()
+
+            if self.subsampleCti is None:
+                return rangeFunction()
+            else:
+                return rangeFunction(self.subsampleCti.configValue)
 
 
     def _updateTargetFromNode(self):
@@ -352,7 +378,7 @@ class AbstractRangeCti(GroupCti):
         targetRange = self.calculateRange()
         #logger.debug("axisRange: {}, padding={}".format(targetRange, padding))
         if not np.all(np.isfinite(targetRange)):
-            logger.warn("New target range is not finite. Plot range not updated")
+            logger.debug("New target range is not finite. Plot range not updated")
             return
 
         self.setTargetRange(targetRange, padding=padding)
@@ -448,6 +474,8 @@ class PgAxisRangeCti(AbstractRangeCti):
 
 class PgHistLutColorRangeCti(AbstractRangeCti):
     """ Configuration tree item is linked to the HistogramLUTItem range.
+
+        Used in the old imageplot2d.py
     """
     def __init__(self, histLutItem, autoRangeFunctions=None, nodeName='color range', expanded=True):
         """ Constructor.
@@ -490,8 +518,135 @@ class PgHistLutColorRangeCti(AbstractRangeCti):
 
 
 
+class PgColorLegendCti(AbstractRangeCti):
+    """ Configuration tree item is linked to the HistogramLUTItem range.
+    """
+    def __init__(self, legend, autoRangeFunctions=None, nodeName='color range', expanded=True):
+        """ Constructor.
+            The target axis is specified by viewBox and axisNumber (0 for x-axis, 1 for y-axis)
+
+            If given, autoRangeFunctions must be a (label to function) dictionary that will be used
+            to populate the (auto range) method ChoiceCti. If not give, the there will not be
+            a method choice and the autorange implemented by PyQtGraph will be used.
+        """
+        super(PgColorLegendCti, self).__init__(
+            autoRangeFunctions=autoRangeFunctions, nodeName=nodeName,
+            expanded=True, paddingDefault=0)
+
+        check_class(legend, ColorLegendItem)
+        self.legend = legend
+
+        self.paddingCti.defaultData = 0
+        # Connect signals
+        self.legend.sigLevelsChanged.connect(self.setAutoRangeOff)
+        self.legend.sigLevelsChanged.connect(self.refreshMinMax)
+
+
+    def _closeResources(self):
+        """ Disconnects signals.
+            Is called by self.finalize when the cti is deleted.
+        """
+        self.legend.sigLevelsChanged.disconnect(self.setAutoRangeOff)
+        self.legend.sigLevelsChanged.disconnect(self.refreshMinMax)
+
+
+    def getTargetRange(self):
+        """ Gets the (color) range of the HistogramLUTItem
+        """
+        return self.legend.getLevels()
+
+
+    def setTargetRange(self, targetRange, padding=None):
+        """ Sets the (color) range of the HistogramLUTItem
+        """
+        self.legend.setLevels(targetRange, padding=padding)
+
+
+
+class PgColorLegendLabelCti(ChoiceCti):
+    """ Configuration tree item that is linked to the axis label of a color legend .
+    """
+    def __init__(self, colorLegendItem, collector,
+                 nodeName='label', defaultData=0, configValues=None):
+        """ Constructor
+            :param colorLegendItem PgColorLegendLabelCti:
+            :param collector: needed to get the collector.rtiInfo
+            :param nodeName: the node name of this config tree item (default = label
+            :param defaultData:
+            :param configValues:
+        """
+        super(PgColorLegendLabelCti, self).__init__(
+            nodeName, editable=True, defaultData=defaultData, configValues=configValues)
+
+        check_class(colorLegendItem, ColorLegendItem)
+        self.colorLegendItem = colorLegendItem
+        self.collector = collector
+
+
+    def _updateTargetFromNode(self):
+        """ Applies the configuration to the target axis it monitors.
+            The axis label will be set to the configValue. If the configValue equals
+            NO_LABEL_STR, the label will be hidden.
+        """
+        if self.configValue == NO_LABEL_STR:
+            self.colorLegendItem.setLabel(None)
+        else:
+            rtiInfo = self.collector.rtiInfo
+            self.colorLegendItem.setLabel(self.configValue.format(**rtiInfo))
+
+
+
+class PgShowHistCti(BoolCti):
+    """ BoolCti that shows/hides the histogram of a color bar.
+    """
+    def __init__(self, colorLegendItem, nodeName='show histogram', defaultData=True):
+        """ Constructor.
+            The target axis is specified by viewBox and axisNumber (0 for x-axis, 1 for y-axis)
+        """
+        super(PgShowHistCti, self).__init__(nodeName, defaultData=defaultData)
+
+        check_class(colorLegendItem, ColorLegendItem)
+        self.colorLegendItem = colorLegendItem
+
+
+    def _updateTargetFromNode(self):
+        """ Applies the configuration to its target axis
+        """
+        self.colorLegendItem.showHistogram(self.configValue)
+
+
+class PgShowDragLinesCti(BoolCti):
+    """ BoolCti that shows/hides the drag lines a color bar.
+    """
+    def __init__(self, colorLegendItem, nodeName='show drag lines',
+                 defaultData=True, expanded=False):
+        """ Constructor.
+            The target axis is specified by viewBox and axisNumber (0 for x-axis, 1 for y-axis)
+        """
+        super(PgShowDragLinesCti, self).__init__(nodeName,
+                                                 defaultData=defaultData, expanded=expanded)
+
+        self.marginCti = self.insertChild(
+            IntCti("margins", 40, minValue=0, maxValue=250, stepSize=5))
+
+        check_class(colorLegendItem, ColorLegendItem)
+        self.colorLegendItem = colorLegendItem
+
+
+    def _updateTargetFromNode(self):
+        """ Applies the configuration to its target axis
+        """
+        try:
+            self.colorLegendItem.showDragLines(self.configValue)
+            self.colorLegendItem.setEdgeMargins(self.marginCti.configValue)
+        except AttributeError as ex:
+            logger.warning("Please update cmlib to latest version. No drag lines: {}".format(ex))
+
+
 class PgGradientEditorItemCti(ChoiceCti):
     """ Lets the user select one of the standard color scales in a GradientEditorItem
+
+        Is not used in the new PgImagePlot2d, only in the one in old_imageplot2d.py.
     """
     def __init__(self, gradientEditorItem, nodeName="color scale", defaultData=-1):
         """ Constructor.
@@ -507,7 +662,7 @@ class PgGradientEditorItemCti(ChoiceCti):
 
 
     def _updateTargetFromNode(self):
-        """ Applies the configuration to its target axis
+        """ Applies the configuration to its target widget
         """
         self.gradientEditorItem.loadPreset(self.configValue)
 
@@ -562,15 +717,13 @@ class PgAxisFlipCti(BoolCti):
 class PgAxisLabelCti(ChoiceCti):
     """ Configuration tree item that is linked to the axis label.
     """
-    NO_LABEL = '-- none --'
-
     def __init__(self, plotItem, axisPosition, collector,
                  nodeName='label', defaultData=0, configValues=None):
         """ Constructor
             :param plotItem:
             :param axisPosition: 'left', 'right', 'bottom', 'top'
             :param collector: needed to get the collector.rtiInfo
-            :param nodeName: the nod name of this config tree item (default = label
+            :param nodeName: the node name of this config tree item (default = label
             :param defaultData:
             :param configValues:
         """
@@ -586,11 +739,11 @@ class PgAxisLabelCti(ChoiceCti):
     def _updateTargetFromNode(self):
         """ Applies the configuration to the target axis it monitors.
             The axis label will be set to the configValue. If the configValue equals
-             PgAxisLabelCti.NO_LABEL, the label will be hidden.
+             NO_LABEL_STR, the label will be hidden.
         """
         rtiInfo = self.collector.rtiInfo
         self.plotItem.setLabel(self.axisPosition, self.configValue.format(**rtiInfo))
-        self.plotItem.showLabel(self.axisPosition, self.configValue != self.NO_LABEL)
+        self.plotItem.showLabel(self.axisPosition, self.configValue != NO_LABEL_STR)
 
 
 
@@ -619,6 +772,7 @@ class PgAxisShowCti(BoolCti):
         self.plotItem.showAxis(self.axisPosition, show=self.configValue) # Seems not to work
 
 
+
 class PgAxisLogModeCti(BoolCti):
     """ BoolCti that toggles an axis between logarithmic vs linear mode.
     """
@@ -684,7 +838,10 @@ class PgAxisCti(GroupCti):
 
 class PgPlotDataItemCti(GroupCti):
     """ Configuration tree item for a PyQtGraph plot data item.
-        It allows for configuraing a line style, color and symbols.
+        It allows for configuring a line style, color and symbols.
+
+        Note that using a line width > 1 in comination with anti-aliassing may be slow.
+        See https://github.com/pyqtgraph/pyqtgraph/issues/533
     """
     def __init__(self, nodeName="pen",  defaultData=None, expanded=True):
         """ Constructor.
@@ -747,3 +904,175 @@ class PgPlotDataItemCti(GroupCti):
                                        symbolPen=symbolPen, symbolBrush=symbolBrush)
         return plotDataItem
 
+
+
+class PgColorMapCti(AbstractCti):
+    """ Lets the user select one of color maps of the color map library
+    """
+    SUB_SAMPLING_OFF = 1
+
+    def __init__(self, colorLegendItem,
+                 nodeName="color map", defaultData=DEFAULT_COLOR_MAP, expanded=False):
+        """ Constructor.
+
+            Stores a color map as data.
+
+            :param defaultData: the default index in the combobox that is used for editing
+        """
+        check_class(colorLegendItem, ArgosColorLegendItem)
+        self.colorLegendItem = colorLegendItem
+        self.cmLibModel = CmLibModelSingleton.instance()
+
+        # grey scale color map for when no color map is selected.
+        lutRgba = np.outer(np.arange(256, dtype=np.uint8), np.array([1, 1, 1, 1], dtype=np.uint8))
+        lutRgba[:,3] = 255
+        self.greyScaleColorMap = ColorMap(CmMetaData("-- none --"), CatalogMetaData("Argos"))
+        self.greyScaleColorMap.set_rgba_uint8_array(lutRgba)
+
+        super(PgColorMapCti, self).__init__(nodeName, defaultData, expanded=expanded)
+
+        self.reverseCti = self.insertChild(
+            BoolCti("reversed", False))
+
+        self.subSampleCti = self.insertChild(
+            IntCti("subsample", self.SUB_SAMPLING_OFF, specialValueText="off",
+                   minValue=self.SUB_SAMPLING_OFF, maxValue=64, stepSize=1)
+        )
+
+
+    def _enforceDataType(self, data):
+        """ Converts to int so that this CTI always stores that type.
+        """
+        if data is None:
+            return self.greyScaleColorMap
+        elif isinstance(data, ColorMap):
+            return data
+        else:
+            return self.cmLibModel.getColorMapByKey(data)
+
+
+    @property
+    def configValue(self):
+        """ The currently selected configValue
+
+            :rtype: ColorMap
+        """
+        return self.data
+
+
+    def _updateTargetFromNode(self):
+        """ Applies the configuration to its target axis.
+            Sets the image item's lookup table to the LUT of the selected color map.
+        """
+        lut = self.data.rgb_uint8_array
+
+        targetSize = self.subSampleCti.configValue
+        if targetSize > self.SUB_SAMPLING_OFF:
+            sourceSize, _ = lut.shape
+            subIdx = np.round(np.linspace(0, sourceSize-1, targetSize)).astype(np.uint)
+            lut = lut[subIdx, :]
+
+        if self.reverseCti.configValue:
+            lut = np.flipud(lut)
+
+        self.colorLegendItem.setLut(lut)
+
+
+    def _dataToString(self, data):
+        """ Conversion function used to convert the (default)data to the display value.
+        """
+        return "" if data is None else data.meta_data.pretty_name
+
+
+    @property
+    def decoration(self):
+        """ Returns a pixmap of the color map to show as icon
+        """
+        return makeColorBarPixmap(
+            self.data,
+            width=self.cmLibModel.iconBarWidth * 0.65,
+            height=self.cmLibModel.iconBarHeight * 0.65,
+            drawBorder=self.cmLibModel.drawIconBarBorder)
+
+
+    def _nodeMarshall(self):
+        """ Returns the non-recursive marshalled value of this CTI. Is called by marshall()
+        """
+        return self.data.key
+
+
+    def _nodeUnmarshall(self, key):
+        """ Initializes itself non-recursively from data. Is called by unmarshall()
+        """
+        self.data = self.cmLibModel.getColorMapByKey(key)
+
+
+    def createEditor(self, delegate, parent, option):
+        """ Creates a ChoiceCtiEditor.
+            For the parameters see the AbstractCti constructor documentation.
+        """
+        return PgColorMapCtiEditor(self, delegate, parent=parent)
+
+
+
+class PgColorMapCtiEditor(AbstractCtiEditor):
+    """ A CtiEditor which contains a QCombobox for editing ChoiceCti objects.
+    """
+    def __init__(self, cti, delegate, parent=None):
+        """ See the AbstractCtiEditor for more info on the parameters
+        """
+        super(PgColorMapCtiEditor, self).__init__(cti, delegate, parent=parent)
+
+        selectionWidget = ColorSelectionWidget(self.cti.cmLibModel)
+        setWidgetSizePolicy(selectionWidget, QtWidgets.QSizePolicy.Expanding, None)
+
+        selectionWidget.sigColorMapHighlighted.connect(self.onColorMapHighlighted)
+        selectionWidget.sigColorMapChanged.connect(self.onColorMapChanged)
+        self.selectionWidget = self.addSubEditor(selectionWidget, isFocusProxy=True)
+        self.comboBox = self.selectionWidget.comboBox
+
+
+    def finalize(self):
+        """ Is called when the editor is closed. Disconnect signals.
+        """
+        logger.debug("PgColorMapCtiEditor.finalize")
+        self.selectionWidget.sigColorMapChanged.disconnect(self.onColorMapChanged)
+        self.selectionWidget.sigColorMapHighlighted.disconnect(self.onColorMapHighlighted)
+        super(PgColorMapCtiEditor, self).finalize()
+
+
+    def setData(self, data):
+        """ Provides the main editor widget with a data to manipulate.
+        """
+        if data is None:
+            logger.warning("No color map to select")
+        else:
+            self.selectionWidget.setColorMapByKey(data.key)
+
+
+    def getData(self):
+        """ Gets data from the editor widget.
+        """
+        return self.selectionWidget.getCurrentColorMap()
+
+
+    @QtSlot(ColorMap)
+    def onColorMapHighlighted(self, colorMap):
+        """ Is called when the user highlights an item in the combo box or dialog.
+
+            The item's index is passed.
+            Note that this signal is sent even when the choice is not changed.
+        """
+        logger.debug("onColorMapHighlighted({})".format(colorMap))
+        self.cti.data = colorMap
+        self.cti.updateTarget()
+
+
+    @QtSlot(ColorMap)
+    def onColorMapChanged(self, index):
+        """ Is called when the user chooses an item in the combo box. The item's index is passed.
+            Note that this signal is sent even when the choice is not changed.
+        """
+        logger.debug("onColorMapChanged")
+        self.delegate.commitData.emit(self)
+
