@@ -33,6 +33,7 @@ from argos.info import DEBUGGING
 from argos.inspector.abstract import AbstractInspector
 from argos.qt import Qt, QtCore, QtGui, QtWidgets
 from argos.widgets.constants import MONO_FONT, FONT_SIZE
+from argos.widgets.argostableview import ArgosTableView
 from argos.utils.cls import check_class, check_is_a_string
 from argos.utils.cls import to_string, is_an_array
 from argos.utils.misc import is_quoted
@@ -63,7 +64,7 @@ def makeReplacementField(formatSpec, altFormatSpec='', testValue=None):
             https://docs.python.org/3/library/string.html#format-string-syntax
             https://docs.python.org/3/library/string.html#format-specification-mini-language
 
-        If the formatSpec does not contain a a color or exclamation mark, a colon is prepended.
+        If the formatSpec does not contain a colon or exclamation mark, a colon is prepended.
 
         If the formatSpec starts and end in quotes (single or double) only the quotes are removed,
         no curly braces or colon charactes are added. This allows users to define a format spec.
@@ -208,16 +209,25 @@ class TableInspectorCti(MainGroupCti):
             self.autoColWidthCti.enable = True
 
 
+    def resetRangesToDefault(self):
+        """ Resets columns auto height and width
+        """
+        self.defaultColWidthCti.resetToDefault()
+        self.defaultRowHeightCti.resetToDefault()
+        self.autoColWidthCti.resetToDefault()
+        self.autoRowHeightCti.resetToDefault()
+
+
 
 class TableInspector(AbstractInspector):
-    """ Shows the sliced array in a table.
+    """ Displays the array data in a table.
     """
     def __init__(self, collector, parent=None):
 
         super(TableInspector, self).__init__(collector, parent=parent)
 
         self.model = TableInspectorModel(parent=self)
-        self.tableView = QtWidgets.QTableView()
+        self.tableView = ArgosTableView()
         self.contentsLayout.addWidget(self.tableView)
         self.tableView.setModel(self.model)
         self.tableView.setSelectionMode(QtWidgets.QTableView.ContiguousSelection)
@@ -271,6 +281,10 @@ class TableInspector(AbstractInspector):
         """
         logger.debug("TableInspector._drawContents: {}".format(self))
 
+        # If auto-reset is true, reset config complete or partially, depending on the mode.
+        if self._resetRequired(reason, initiator):
+            self.resetConfig()
+
         oldTableIndex = self.tableView.currentIndex()
         if oldTableIndex.isValid():
             selectionWasValid = True # Defensive programming, keep old value just in case
@@ -281,6 +295,12 @@ class TableInspector(AbstractInspector):
             oldRow = 0
             oldCol = 0
 
+        # Temporarily set header sizes fixed when populating the model (mayy be slow otherwise)
+        verHeader = self.tableView.verticalHeader()
+        horHeader = self.tableView.horizontalHeader()
+        verHeader.setSectionResizeMode(QtWidgets.QHeaderView.Fixed)
+        horHeader.setSectionResizeMode(QtWidgets.QHeaderView.Fixed)
+
         self.model.updateState(self.collector.getSlicedArray(),
                                self.collector.rtiInfo,
                                self.configValue('separate fields'))
@@ -310,7 +330,6 @@ class TableInspector(AbstractInspector):
         # Update the model font from the font config item (will call self.setFont)
         self.config.updateTarget()
 
-        verHeader = self.tableView.verticalHeader()
         if (self.config.autoRowHeightCti.configValue
             and self.model.rowCount() < RESET_HEADERS_AT_SIZE):
 
@@ -321,7 +340,6 @@ class TableInspector(AbstractInspector):
             else:
                 # ResizeToContents can be very slow because it gets all rows.
                 # Work around: resize only first row and columns and apply this to all rows/cols
-                # TODO: perhaps use SizeHintRole?
                 logger.warning("Performance work around: for tables with more than 1000 cells " +
                                "only the current row is resized and the others use that size.")
                 verHeader.setSectionResizeMode(QtWidgets.QHeaderView.Interactive)
@@ -333,7 +351,6 @@ class TableInspector(AbstractInspector):
             verHeader.setDefaultSectionSize(self.config.defaultRowHeightCti.configValue)
             verHeader.reset()
 
-        horHeader = self.tableView.horizontalHeader()
         if (self.config.autoColWidthCti.configValue
             and self.model.columnCount() < RESET_HEADERS_AT_SIZE):
 
@@ -344,7 +361,6 @@ class TableInspector(AbstractInspector):
             else:
                 # ResizeToContents can be very slow because it gets all rows.
                 # Work around: resize only first row and columns and apply this to all rows/cols
-                # TODO: perhaps use SizeHintRole?
                 logger.warning("Performance work around: for tables with more than 1000 cells " +
                                "only the current column is resized and the others use that size.")
                 horHeader.setSectionResizeMode(QtWidgets.QHeaderView.Interactive)
@@ -406,6 +422,7 @@ class TableInspectorModel(QtCore.QAbstractTableModel):
 
             Will be called from the tableInspector._drawContents.
         """
+        logger.debug("TableInspectorModel.updateState called")
         self.beginResetModel()
         try:
             # The sliced array can be a masked array or a (regular) numpy array.
