@@ -26,9 +26,6 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
     HEADERS = tuple('<first column') # override in descendants
     COL_DECORATION = None   # Column number that contains the decoration. None for no icons
 
-    # Can be connected to a QTreeView.update() as to update a single cell
-    #update = QtSignal(QtCore.QModelIndex) # Not used
-
     # Signal emitted when an item has been changed (e.g. by setData). This can be used to update
     # other widgets, whereas the dataChanged signal is used to update the treeView only. This
     # separation is introduced to break circular updates that would occur when the other widgets
@@ -37,7 +34,11 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
     # will see this is argos specific and thus find this definition more easily.
     sigItemChanged = QtSignal(BaseTreeItem)
 
+    # Emitted in removeAllChildrenAtIndex.
+    sigAllChildrenRemovedAtIndex = QtSignal(QtCore.QModelIndex)
+
     def __init__(self, parent=None):
+
         """ Constructor
         """
         super(BaseTreeModel, self).__init__(parent)
@@ -47,16 +48,32 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         # The root item also is returned by getItem in case of an invalid index.
         # Finally, it is used to store the header data.
         #self._horizontalHeaders = [header for header in headers]
-        self._invisibleRootItem = BaseTreeItem(nodeName=INVISIBLE_ROOT_NAME)
+        self._invisibleRootTreeItem = BaseTreeItem(nodeName=INVISIBLE_ROOT_NAME)
 
         self.cellSizeHint = TREE_CELL_SIZE_HINT # Default cell size, can be overridden by items
 
 
     @property
-    def invisibleRootItem(self):
+    def invisibleRootTreeItem(self):
         """ Returns the invisible root item, which contains no actual data
         """
-        return self._invisibleRootItem
+        return self._invisibleRootTreeItem
+
+
+    def setInvisibleRootItem(self, treeItem=None):
+         """ SetsReplaces the invisible root item, which contains no actual data
+
+             If baseRti is None it is set to a place holder BaseTreeItem.
+             Note that this resets the model
+         """
+         treeItem = BaseTreeItem(nodeName=INVISIBLE_ROOT_NAME) if treeItem is None else treeItem
+
+         self.beginResetModel()
+         try:
+             self._invisibleRootTreeItem = treeItem
+             self._invisibleRootTreeItem.model = self
+         finally:
+             self.endResetModel()
 
 
     @property
@@ -87,7 +104,7 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         """
         try:
             if index.isValid():
-                item = self.getItem(index, altItem=self.invisibleRootItem)
+                item = self.getItem(index, altItem=self.invisibleRootTreeItem)
                 return self.itemData(item, index.column(), role=role)
             else:
                 return None
@@ -106,9 +123,6 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         """ Returns the data stored under the given role for the item. O
             The column parameter may be used to differentiate behavior per column.
 
-            The default implementation does nothing. Descendants should typically override this
-            function instead of data()
-
             Note: If you do not have a value to return, return an invalid QVariant instead of
             returning 0. (This means returning None in Python)
         """
@@ -173,23 +187,23 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
 #                     .format(parentIndex.row(), parentIndex.column(), parentIndex.isValid(),
 #                             parentIndex.isValid() and parentIndex.column() != 0))
 
-        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootItem)
+        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootTreeItem)
         #logger.debug("    Getting row {} from parentItem: {}".format(row, parentItem))
 
         if not (0 <= row < parentItem.nChildren()):
             # Can happen when deleting the last child.
-            #logger.warn("Index row {} invalid for parent item: {}".format(row, parentItem))
+            #logger.warning("Index row {} invalid for parent item: {}".format(row, parentItem))
             return QtCore.QModelIndex()
 
         if not (0 <= column < self.columnCount()):
-            #logger.warn("Index column {} invalid for parent item: {}".format(column, parentItem))
+            #logger.warning("Index column {} invalid for parent item: {}".format(column, parentItem))
             return QtCore.QModelIndex()
 
         childItem = parentItem.child(row)
         if childItem:
             return self.createIndex(row, column, childItem)
         else:
-            logger.warn("No child item found at row {} for parent item: {}".format(row, parentItem))
+            logger.warning("No child item found at row {} for parent item: {}".format(row, parentItem))
             return QtCore.QModelIndex()
 
 
@@ -208,10 +222,10 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         if not index.isValid():
             return QtCore.QModelIndex()
 
-        childItem = self.getItem(index, altItem=self.invisibleRootItem)
+        childItem = self.getItem(index, altItem=self.invisibleRootTreeItem)
         parentItem = childItem.parentItem
 
-        if parentItem == self.invisibleRootItem:
+        if parentItem == self.invisibleRootTreeItem:
             return QtCore.QModelIndex()
 
         return self.createIndex(parentItem.childNumber(), 0, parentItem)
@@ -224,7 +238,7 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
             Note: When implementing a table based model, rowCount() should return 0 when the parent
             is valid.
         """
-        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootItem)
+        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootTreeItem)
         return parentItem.nChildren()
 
 
@@ -232,7 +246,7 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         """ Returns true if parent has any children; otherwise returns false.
             Use rowCount() on the parent to find out the number of children.
         """
-        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootItem)
+        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootTreeItem)
         return parentItem.hasChildren()
 
 
@@ -248,7 +262,7 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         if role != Qt.CheckStateRole and role != Qt.EditRole:
             return False
 
-        treeItem = self.getItem(index, altItem=self.invisibleRootItem)
+        treeItem = self.getItem(index, altItem=self.invisibleRootTreeItem)
         try:
             result = self.setItemData(treeItem, index.column(), value, role=role)
             if result:
@@ -265,13 +279,12 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
 
         except Exception as ex:
             # When does this still happen? Can we remove it?
-            logger.warn("Unable to set data: {}".format(ex))
+            logger.warning("Unable to set data: {}".format(ex))
             if DEBUGGING:
                 raise
             return False
 
 
-
     def setItemData(self, item, column, value, role=Qt.EditRole):
         """ Sets the role data for the item at index to value.
 
@@ -300,8 +313,8 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
 
 
     # Originally from the editabletreemodel example but added the altItem parameter to force
-    # callers to specify request the invisibleRootItem as alternative in case of an invalid index.
-    # TODO: rename to ItemFromIndex to be consistent with QStandardItemModel?
+    # callers to specify request the an alternative in case of an invalid index.
+    # The equivalent function in the QStandardItemModel is itemFromIndex.
     def getItem(self, index, altItem=None):
         """ Returns the TreeItem for the given index. Returns the altItem if the index is invalid.
         """
@@ -310,7 +323,6 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
             if item:
                 return item
 
-        #return altItem if altItem is not None else self.invisibleRootItem # TODO: remove
         return altItem
 
 
@@ -323,7 +335,7 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         if parentIndex is None:
             parentIndex=QtCore.QModelIndex()
 
-        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootItem)
+        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootTreeItem)
 
         nChildren = parentItem.nChildren()
         if position is None:
@@ -367,7 +379,10 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         finally:
             self.endRemoveRows()
 
-        logger.debug("removeAllChildrenAtIndex completed")
+        logger.debug("Emitting sigChildrenRemovedAtIndex")
+        self.sigAllChildrenRemovedAtIndex.emit(parentIndex)
+
+        logger.debug("sigAllChildrenRemovedAtIndex completed")
 
 
     def deleteItemAtIndex(self, itemIndex):
@@ -382,7 +397,7 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
         logger.debug("deleteItemAtIndex: removing {}".format(item))
 
         parentIndex = itemIndex.parent()
-        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootItem)
+        parentItem = self.getItem(parentIndex, altItem=self.invisibleRootTreeItem)
         row = itemIndex.row()
         self.beginRemoveRows(parentIndex, row, row)
         try:
@@ -486,7 +501,7 @@ class BaseTreeModel(QtCore.QAbstractItemModel):
 
         if startIndex is None or path.startswith('/'):
             startIndex = QtCore.QModelIndex()
-            startItem = self.invisibleRootItem
+            startItem = self.invisibleRootTreeItem
         else:
             startItem = self.getItem(startIndex, None)
 
