@@ -18,22 +18,25 @@
 """ Repository Tree Items (RTIs) for Exdir data.
 
 """
+from __future__ import print_function, absolute_import
 
 import logging, os
 import collections
 import exdir
 import numpy as np
 
-from argos.repo.iconfactory import RtiIconFactory
+from argos.repo.iconfactory import RtiIconFactory, ICON_COLOR_UNDEF
 from argos.repo.baserti import BaseRti
 from argos.repo.filesytemrtis import createRtiFromFileName
-from argos.utils.cls import to_string, check_class, is_an_array
+from argos.repo.rtiplugins.hdf5 import dimNamesFromDataset  # We can reuse it, the exdir module follows the h5py API.
+from argos.utils.cls import check_class, is_an_array
+from argos.utils.defs import DIM_TEMPLATE, SUB_DIM_TEMPLATE
 from argos.utils.masks import maskedEqual
 
 
 logger = logging.getLogger(__name__)
 
-ICON_COLOR_EXDIR = '#00BBFF'
+
 
 
 def dataSetElementType(exdirDataset):
@@ -95,12 +98,12 @@ def dataSetMissingValue(exdirDataset):
 
 
 def flattenDict(d, parent_key='', sep='/'):
-    """ Returns a flatten dictionary given a nested dictionary. 
+    """ Returns a flatten dictionary given a nested dictionary.
 
-        The nested keys are be separated by sep. 
+        The nested keys are be separated by sep.
     """
     items = []
-    if isinstance(d,list): d = {str(i) : v for i, v in enumerate(d)} 
+    if isinstance(d,list): d = {str(i) : v for i, v in enumerate(d)}
     for k, v in d.items():
         new_key = parent_key + sep + k if parent_key else k
         if isinstance(v, collections.MutableMapping):
@@ -116,12 +119,12 @@ class ExdirScalarRti(BaseRti):
 
     """
     _defaultIconGlyph = RtiIconFactory.SCALAR
-    _defaultIconColor = ICON_COLOR_EXDIR
 
-    def __init__(self, exdirDataset, nodeName='', fileName=''):
+    def __init__(self, exdirDataset, nodeName='', fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(ExdirScalarRti, self).__init__(nodeName = nodeName, fileName=fileName)
+        super(ExdirScalarRti, self).__init__(
+            nodeName=nodeName, fileName=fileName, iconColor=iconColor)
         check_class(exdirDataset, exdir.Dataset)
         self._exdirDataset = exdirDataset
 
@@ -189,13 +192,12 @@ class ExdirFieldRti(BaseRti):
     """ Repository Tree Item (RTI) that contains a field in a structured HDF-5 variable.
     """
     _defaultIconGlyph = RtiIconFactory.FIELD
-    _defaultIconColor = ICON_COLOR_EXDIR
 
-    def __init__(self, exdirDataset, nodeName, fileName=''):
+    def __init__(self, exdirDataset, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor.
             The name of the field must be given to the nodeName parameter.
         """
-        super(ExdirFieldRti, self).__init__(nodeName, fileName=fileName)
+        super(ExdirFieldRti, self).__init__(nodeName, fileName=fileName, iconColor=iconColor)
         check_class(exdirDataset, exdir.Dataset)
         self._exdirDataset = exdirDataset
 
@@ -280,7 +282,7 @@ class ExdirFieldRti(BaseRti):
         """ Returns a list with the dimension names of the underlying NCDF variable
         """
         nSubDims = len(self._subArrayShape)
-        subArrayDims = ['SubDim{}'.format(dimNr) for dimNr in range(nSubDims)]
+        subArrayDims = [SUB_DIM_TEMPLATE.format(dimNr) for dimNr in range(nSubDims)]
         return dimNamesFromDataset(self._exdirDataset) + subArrayDims
 
 
@@ -316,25 +318,25 @@ class ExdirFieldRti(BaseRti):
             return value
 
 
+
 class ExdirDatasetRti(BaseRti):
     """ Repository Tree Item (RTI) that contains a HDF5 dataset.
 
         This includes dimenions scales, which are then displayed with a different icon.
     """
     _defaultIconGlyph = RtiIconFactory.ARRAY # the iconGlyph property is overridden below
-    _defaultIconColor = ICON_COLOR_EXDIR
 
-    def __init__(self, exdirDataset, nodeName, fileName=''):
+    def __init__(self, exdirDataset, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(ExdirDatasetRti, self).__init__(nodeName, fileName=fileName)
+        super(ExdirDatasetRti, self).__init__(nodeName, fileName=fileName, iconColor=iconColor)
         check_class(exdirDataset, exdir.Dataset)
         self._exdirDataset = exdirDataset
         self._isStructured = bool(self._exdirDataset.dtype.names)
 
-
-        fileNames = os.listdir(os.path.abspath(self._exdirDataset.directory))
-        absFileNames = [os.path.join(self._exdirDataset.directory, fn) for fn in fileNames]
+        exdirDir = str(self._exdirDataset.directory) # convert to string for Python 3.5
+        fileNames = os.listdir(os.path.abspath(exdirDir))
+        absFileNames = [os.path.join(exdirDir, fn) for fn in fileNames]
         self._hasRaws = sum([os.path.isdir(f) for f in absFileNames]) > 0
 
     def hasChildren(self):
@@ -405,34 +407,33 @@ class ExdirDatasetRti(BaseRti):
         if self._isStructured:
             for fieldName in self._exdirDataset.dtype.names:
                 childItems.append(ExdirFieldRti(self._exdirDataset, nodeName=fieldName,
-                                               fileName=self.fileName))
-
+                                                fileName=self.fileName, iconColor=self.iconColor))
 
         # Add raw directories
         if self._hasRaws:
-            fileNames = os.listdir(os.path.abspath(self._exdirDataset.directory))
-            absFileNames = [os.path.join(self._exdirDataset.directory, fn) for fn in fileNames]
+            exdirDir = str(self._exdirDataset.directory) # convert to string for Python 3.5
+            fileNames = os.listdir(os.path.abspath(exdirDir))
+            absFileNames = [os.path.join(exdirDir, fn) for fn in fileNames]
 
             for fileName, absFileName in zip(fileNames, absFileNames):
                 if os.path.isdir(absFileName) and not fileName.startswith('.'):
-                    childItems.append(ExdirRawRti(self._exdirDataset.require_raw(fileName), nodeName=fileName,
-                                                    fileName=self.fileName))
-
-
+                    childItems.append(ExdirRawRti(
+                        self._exdirDataset.require_raw(fileName), nodeName=fileName,
+                        fileName=self.fileName, iconColor=self.iconColor))
 
         return childItems
 
 
+
 class ExdirRawRti(BaseRti):
     """ Repository Tree Item (RTI) that contains an Exdir Raw.
     """
     _defaultIconGlyph = RtiIconFactory.FOLDER
-    _defaultIconColor = ICON_COLOR_EXDIR
 
-    def __init__(self, exdirRaw, nodeName, fileName=''):
+    def __init__(self, exdirRaw, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(ExdirRawRti, self).__init__(nodeName, fileName=fileName)
+        super(ExdirRawRti, self).__init__(nodeName, fileName=fileName, iconColor=iconColor)
         check_class(exdirRaw, exdir.Raw, allow_none=True)
 
         self._exdirRaw = exdirRaw
@@ -445,10 +446,10 @@ class ExdirRawRti(BaseRti):
         assert self.canFetchChildren(), "canFetchChildren must be True"
 
         childItems = []
-
-        absPaths = os.path.abspath(self._exdirRaw.directory)
+        exdirDir = str(self._exdirRaw.directory) # convert to string for Python 3.5
+        absPaths = os.path.abspath(exdirDir)
         fileNames = os.listdir(absPaths)
-        absFileNames = [os.path.join(self._exdirRaw.directory, fn) for fn in fileNames]
+        absFileNames = [os.path.join(exdirDir, fn) for fn in fileNames]
 
         for fileName, absFileName in zip(fileNames, absFileNames):
             if not fileName.startswith('.'):
@@ -458,16 +459,16 @@ class ExdirRawRti(BaseRti):
         return childItems
 
 
+
 class ExdirGroupRti(BaseRti):
     """ Repository Tree Item (RTI) that contains an Exdir group.
     """
     _defaultIconGlyph = RtiIconFactory.FOLDER
-    _defaultIconColor = ICON_COLOR_EXDIR
 
-    def __init__(self, exdirGroup, nodeName, fileName=''):
+    def __init__(self, exdirGroup, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(ExdirGroupRti, self).__init__(nodeName, fileName=fileName)
+        super(ExdirGroupRti, self).__init__(nodeName, fileName=fileName, iconColor=iconColor)
         check_class(exdirGroup, exdir.Group, allow_none=True)
 
         self._exdirGroup = exdirGroup
@@ -491,43 +492,45 @@ class ExdirGroupRti(BaseRti):
         for childName, exdirChild in self._exdirGroup.items():
             if isinstance(exdirChild, exdir.Group):
                 childItems.append(ExdirGroupRti(exdirChild, nodeName=childName,
-                                               fileName=self.fileName))
+                                                fileName=self.fileName, iconColor=self.iconColor))
 
             elif isinstance(exdirChild, exdir.Raw):
                 childItems.append(ExdirRawRti(exdirChild, nodeName=childName,
-                                               fileName=self.fileName))
-            
+                                              fileName=self.fileName, iconColor=self.iconColor))
+
             elif isinstance(exdirChild, exdir.Dataset):
                 if len(exdirChild.shape) == 0:
-                    childItems.append(ExdirScalarRti(exdirChild, nodeName=childName,
-                                                    fileName=self.fileName))
+                    childItems.append(ExdirScalarRti(
+                        exdirChild, nodeName=childName, fileName=self.fileName,
+                        iconColor=self.iconColor))
                 else:
-                    childItems.append(ExdirDatasetRti(exdirChild, nodeName=childName,
-                                                     fileName=self.fileName))
+                    childItems.append(ExdirDatasetRti(
+                        exdirChild, nodeName=childName, fileName=self.fileName,
+                        iconColor=self.iconColor))
             else:
-                logger.warn("Ignored {}. It has an unexpected Exdir type: {}"
+                logger.warning("Ignored {}. It has an unexpected Exdir type: {}"
                             .format(childName, type(exdir)))
 
         return childItems
 
 
+
 class ExdirFileRti(ExdirGroupRti):
     """ Reads an Exdir file using the exdir package.
     """
     _defaultIconGlyph = RtiIconFactory.FILE
-    _defaultIconColor = ICON_COLOR_EXDIR
 
-    def __init__(self, nodeName, fileName=''):
+    def __init__(self, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(ExdirFileRti, self).__init__(None, nodeName, fileName=fileName)
+        super(ExdirFileRti, self).__init__(None, nodeName, fileName=fileName, iconColor=iconColor)
         self._checkFileExists()
 
     def _openResources(self):
         """ Opens the root Dataset.
         """
         logger.info("Opening: {}".format(self._fileName))
-        self._exdirGroup = exdir.File(self._fileName)
+        self._exdirGroup = exdir.File(self._fileName, mode='r')
 
 
     def _closeResources(self):
@@ -535,4 +538,4 @@ class ExdirFileRti(ExdirGroupRti):
         """
         logger.info("Closing: {}".format(self._fileName))
         self._exdirGroup.close()
-        self._exdirGroup = None
\ No newline at end of file
+        self._exdirGroup = None
