@@ -20,23 +20,23 @@
     It uses the h5py package to open HDF-5 files.
     See http://www.h5py.org/
 """
+from __future__ import absolute_import
 
 import logging, os
 import h5py
-import numpy as np
 
 
-from argos.repo.iconfactory import RtiIconFactory
+from argos.repo.iconfactory import RtiIconFactory, ICON_COLOR_UNDEF
 from argos.repo.baserti import BaseRti
 from argos.utils.cls import to_string, check_class, is_an_array
+from argos.utils.defs import DIM_TEMPLATE, SUB_DIM_TEMPLATE, CONTIGUOUS
 from argos.utils.masks import maskedEqual
 
 logger = logging.getLogger(__name__)
 
-ICON_COLOR_H5PY = '#00EE88'
 
 
-def dimNamesFromDataset(h5Dataset):
+def dimNamesFromDataset(h5Dataset, useBaseName=True):
     """ Constructs the dimension names given a h5py dataset.
 
         First looks in the dataset's dimension scales to see if it refers to another
@@ -46,20 +46,25 @@ def dimNamesFromDataset(h5Dataset):
     dimNames = [] # TODO: cache?
     for dimNr, dimScales in enumerate(h5Dataset.dims):
         if len(dimScales) == 0:
-            dimNames.append('Dim{}'.format(dimNr))
-        elif len(dimScales) == 1:
+            dimNames.append(DIM_TEMPLATE.format(dimNr))
+        else:
             dimScaleLabel, dimScaleDataset = dimScales.items()[0]
             path = dimScaleDataset.name
             if path:
-                dimNames.append(os.path.basename(path))
+                dimName = os.path.basename(path) if useBaseName else path
             elif dimScaleLabel: # This could potentially be long so it's our second choice
-                dimNames.append(dimScaleLabel)
+                dimName = dimScaleLabel
             else:
-                dimNames.append('Dim{}'.format(dimNr))
-        else:
-            # TODO: multiple scales for this dimension. What to do?
-            logger.warn("More than one dimension scale found: {!r}".format(dimScales))
-            dimNames.append('Dim{}'.format(dimNr)) # For now, just number them
+                dimName = DIM_TEMPLATE.format(dimNr)
+
+            if len(dimScales) > 1:
+                logger.debug("More than one dimension scale found, using the first {}"
+                             .format(dimName))
+
+            dimNames.append(dimName)
+        # else:
+        #     # TODO: multiple scales for this dimension. What to do?
+        #     dimNames.append(DIM_TEMPLATE.format(dimNr)) # For now, just number them
 
     return dimNames
 
@@ -104,6 +109,26 @@ def dataSetUnit(h5Dataset):
     return ''
 
 
+def attrsToDict(attrs):
+    """ Converts attributes to a dictionary.
+
+        In case an attribute can't be read and raises and exception, the attribute value will be
+        the error message. This will make the attributes more robuts.
+
+        Necessary because, for instacnce, h5py may not be able to read all attributes
+        generated by new versions of the Python netCDF4 library.
+        # See: https://github.com/h5py/h5py/issues/719
+    """
+    result = {}
+    for key in attrs.keys():
+        try:
+            result[key] = attrs[key]
+        except Exception as ex:
+            logger.warning("Unable to read {} attribute: {}".format(key, ex))
+            result[key] = str(ex)
+    return result
+
+
 def dataSetMissingValue(h5Dataset):
     """ Returns the missingData given a HDF-5 dataset
 
@@ -133,12 +158,12 @@ class H5pyScalarRti(BaseRti):
 
     """
     _defaultIconGlyph = RtiIconFactory.SCALAR
-    _defaultIconColor = ICON_COLOR_H5PY
 
-    def __init__(self, h5Dataset, nodeName='', fileName=''):
+    def __init__(self, h5Dataset, nodeName='', fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(H5pyScalarRti, self).__init__(nodeName = nodeName, fileName=fileName)
+        super(H5pyScalarRti, self).__init__(
+            nodeName=nodeName, fileName=fileName, iconColor=iconColor)
         check_class(h5Dataset, h5py.Dataset)
         self._h5Dataset = h5Dataset
 
@@ -160,18 +185,14 @@ class H5pyScalarRti(BaseRti):
         """ Called when using the RTI with an index (e.g. rti[0]).
             The scalar will be wrapped in an array with one element so it can be inspected.
         """
-        array = np.array([self._h5Dataset[()]]) # slice with empty tuple
-        maskedArray = maskedEqual(array, self.missingDataValue)
-
-        assert maskedArray.shape == (1, ), "Scalar wrapper shape mismatch: {}".format(array.shape)
-        return maskedArray[index] # Use the index to ensure the slice has the correct shape
+        return self._h5Dataset[()]
 
 
     @property
     def arrayShape(self):
-        """ Returns the shape of the wrapper array. Will always be the tuple (1, )
+        """ Returns the shape of the wrapper array. Will always be an empty tuple()
         """
-        return (1, )
+        return tuple()
 
 
     @property
@@ -185,7 +206,7 @@ class H5pyScalarRti(BaseRti):
     def attributes(self):
         """ The attributes dictionary.
         """
-        return self._h5Dataset.attrs
+        return attrsToDict(self._h5Dataset.attrs)
 
 
     @property
@@ -207,13 +228,12 @@ class H5pyFieldRti(BaseRti):
     """ Repository Tree Item (RTI) that contains a field in a structured HDF-5 variable.
     """
     _defaultIconGlyph = RtiIconFactory.FIELD
-    _defaultIconColor = ICON_COLOR_H5PY
 
-    def __init__(self, h5Dataset, nodeName, fileName=''):
+    def __init__(self, h5Dataset, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor.
             The name of the field must be given to the nodeName parameter.
         """
-        super(H5pyFieldRti, self).__init__(nodeName, fileName=fileName)
+        super(H5pyFieldRti, self).__init__(nodeName, fileName=fileName, iconColor=iconColor)
         check_class(h5Dataset, h5py.Dataset)
         self._h5Dataset = h5Dataset
 
@@ -229,7 +249,7 @@ class H5pyFieldRti(BaseRti):
         """ The attributes dictionary.
             Returns the attributes of the variable that contains this field.
         """
-        return self._h5Dataset.attrs
+        return attrsToDict(self._h5Dataset.attrs)
 
 
     @property
@@ -241,7 +261,7 @@ class H5pyFieldRti(BaseRti):
 
     def __getitem__(self, index):
         """ Called when using the RTI with an index (e.g. rti[0]).
-            Applies the index on the NCDF variable that contain this field and then selects the
+            Applies the index on the HDF dataset that contain this field and then selects the
             current field. In pseudo-code, it returns: self.ncVar[index][self.nodeName].
 
             If the field itself contains a sub-array it returns:
@@ -285,6 +305,14 @@ class H5pyFieldRti(BaseRti):
         return self._h5Dataset.shape + self._subArrayShape
 
 
+    @property
+    def chunking(self):
+        """ List with chunk sizes if chunked storage is used. Or 'contiguous' for contiguous storage
+        """
+        chunks = self._h5Dataset.chunks
+        return CONTIGUOUS if chunks is None else chunks
+
+
     @property
     def elementTypeName(self):
         """ String representation of the element type.
@@ -295,13 +323,22 @@ class H5pyFieldRti(BaseRti):
 
     @property
     def dimensionNames(self):
-        """ Returns a list with the dimension names of the underlying NCDF variable
+        """ Returns a list with the dimension names of the underlying HDF5 variable
         """
         nSubDims = len(self._subArrayShape)
-        subArrayDims = ['SubDim{}'.format(dimNr) for dimNr in range(nSubDims)]
+        subArrayDims = [SUB_DIM_TEMPLATE.format(dimNr) for dimNr in range(nSubDims)]
         return dimNamesFromDataset(self._h5Dataset) + subArrayDims
 
 
+    @property
+    def dimensionPaths(self):
+        """ Returns a list with the full path names of the dimensions.
+        """
+        nSubDims = len(self._subArrayShape)
+        subArrayDims = [SUB_DIM_TEMPLATE.format(dimNr) for dimNr in range(nSubDims)]
+        return dimNamesFromDataset(self._h5Dataset, useBaseName=False) + subArrayDims
+
+
     @property
     def unit(self):
         """ Returns the unit of the RTI by calling dataSetUnit on the underlying dataset
@@ -341,12 +378,11 @@ class H5pyDatasetRti(BaseRti):
         This includes dimenions scales, which are then displayed with a different icon.
     """
     #_defaultIconGlyph = RtiIconFactory.ARRAY # the iconGlyph property is overridden below
-    _defaultIconColor = ICON_COLOR_H5PY
 
-    def __init__(self, h5Dataset, nodeName, fileName=''):
+    def __init__(self, h5Dataset, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(H5pyDatasetRti, self).__init__(nodeName, fileName=fileName)
+        super(H5pyDatasetRti, self).__init__(nodeName, fileName=fileName, iconColor=iconColor)
         check_class(h5Dataset, h5py.Dataset)
         self._h5Dataset = h5Dataset
         self._isStructured = bool(self._h5Dataset.dtype.names)
@@ -356,12 +392,17 @@ class H5pyDatasetRti(BaseRti):
     def iconGlyph(self):
         """ Shows an Array icon for regular datasets but a dimension icon for dimension scales
         """
-        if self._h5Dataset.attrs.get('CLASS', None) == b'DIMENSION_SCALE':
-            return RtiIconFactory.DIMENSION
-        else:
+        try:
+            if self._h5Dataset.attrs.get('CLASS', None) == b'DIMENSION_SCALE':
+                return RtiIconFactory.DIMENSION
+            else:
+                return RtiIconFactory.ARRAY
+        except OSError as ex:
+            logger.warning("Unable to read 'CLASS' attribute. Assuming regular dataset".format(ex))
             return RtiIconFactory.ARRAY
 
 
+
     def hasChildren(self):
         """ Returns True if the variable has a structured type, otherwise returns False.
         """
@@ -390,6 +431,14 @@ class H5pyDatasetRti(BaseRti):
         return self._h5Dataset.shape
 
 
+    @property
+    def chunking(self):
+        """ List with chunk sizes if chunked storage is used. Or 'contiguous' for contiguous storage
+        """
+        chunks = self._h5Dataset.chunks
+        return CONTIGUOUS if chunks is None else chunks
+
+
     @property
     def elementTypeName(self):
         """ String representation of the element type.
@@ -401,7 +450,7 @@ class H5pyDatasetRti(BaseRti):
     def attributes(self):
         """ The attributes dictionary.
         """
-        return self._h5Dataset.attrs
+        return attrsToDict(self._h5Dataset.attrs)
 
 
     @property
@@ -411,6 +460,13 @@ class H5pyDatasetRti(BaseRti):
         return dimNamesFromDataset(self._h5Dataset) # TODO: cache?
 
 
+    @property
+    def dimensionPaths(self):
+        """ Returns a list with the full path names of the dimensions.
+        """
+        return dimNamesFromDataset(self._h5Dataset, useBaseName=False)
+
+
     @property
     def unit(self):
         """ Returns the unit of the RTI by calling dataSetUnit on the underlying dataset
@@ -437,7 +493,7 @@ class H5pyDatasetRti(BaseRti):
         if self._isStructured:
             for fieldName in self._h5Dataset.dtype.names:
                 childItems.append(H5pyFieldRti(self._h5Dataset, nodeName=fieldName,
-                                               fileName=self.fileName))
+                                               fileName=self.fileName, iconColor=self.iconColor))
 
         return childItems
 
@@ -446,12 +502,11 @@ class H5pyGroupRti(BaseRti):
     """ Repository Tree Item (RTI) that contains a HDF-5 group.
     """
     _defaultIconGlyph = RtiIconFactory.FOLDER
-    _defaultIconColor = ICON_COLOR_H5PY
 
-    def __init__(self, h5Group, nodeName, fileName=''):
+    def __init__(self, h5Group, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(H5pyGroupRti, self).__init__(nodeName, fileName=fileName)
+        super(H5pyGroupRti, self).__init__(nodeName, fileName=fileName, iconColor=iconColor)
         check_class(h5Group, h5py.Group, allow_none=True)
 
         self._h5Group = h5Group
@@ -461,7 +516,7 @@ class H5pyGroupRti(BaseRti):
     def attributes(self):
         """ The attributes dictionary.
         """
-        return self._h5Group.attrs if self._h5Group else {}
+        return attrsToDict(self._h5Group.attrs if self._h5Group else {})
 
 
     def _fetchAllChildren(self):
@@ -474,21 +529,25 @@ class H5pyGroupRti(BaseRti):
 
         for childName, h5Child in self._h5Group.items():
             if isinstance(h5Child, h5py.Group):
-                childItems.append(H5pyGroupRti(h5Child, nodeName=childName,
-                                               fileName=self.fileName))
+                childItems.append(H5pyGroupRti(
+                    h5Child, nodeName=childName,
+                    fileName=self.fileName, iconColor=self.iconColor))
             elif isinstance(h5Child, h5py.Dataset):
-                if len(h5Child.shape) == 0:
-                    childItems.append(H5pyScalarRti(h5Child, nodeName=childName,
-                                                    fileName=self.fileName))
+                # The shape can be None in case of Null datasets.
+                if h5Child.shape is None or len(h5Child.shape) == 0:
+                    childItems.append(H5pyScalarRti(
+                        h5Child, nodeName=childName,
+                        fileName=self.fileName, iconColor=self.iconColor))
                 else:
-                    childItems.append(H5pyDatasetRti(h5Child, nodeName=childName,
-                                                     fileName=self.fileName))
+                    childItems.append(H5pyDatasetRti(
+                        h5Child, nodeName=childName,
+                        fileName=self.fileName, iconColor=self.iconColor))
 
             elif isinstance(h5Child, h5py.Datatype):
                 #logger.debug("Ignored DataType item: {}".format(childName))
                 pass
             else:
-                logger.warn("Ignored {}. It has an unexpected HDF-5 type: {}"
+                logger.warning("Ignored {}. It has an unexpected HDF-5 type: {}"
                             .format(childName, type(h5Child)))
 
         return childItems
@@ -501,18 +560,19 @@ class H5pyFileRti(H5pyGroupRti):
         See http://www.h5py.org/
     """
     _defaultIconGlyph = RtiIconFactory.FILE
-    _defaultIconColor = ICON_COLOR_H5PY
 
-    def __init__(self, nodeName, fileName=''):
+    def __init__(self, nodeName, fileName='', iconColor=ICON_COLOR_UNDEF):
         """ Constructor
         """
-        super(H5pyFileRti, self).__init__(None, nodeName, fileName=fileName)
+        super(H5pyFileRti, self).__init__(None, nodeName, fileName=fileName, iconColor=iconColor)
         self._checkFileExists()
 
     def _openResources(self):
         """ Opens the root Dataset.
         """
         logger.info("Opening: {}".format(self._fileName))
+        if not os.path.isfile(self._fileName):
+            raise OSError("{} does not exist or is not a regular file.".format(self._fileName))
         self._h5Group = h5py.File(self._fileName, 'r')
 
 
